/*
 * Created at regulus on 16-oct-2004 12:53:37
 * as file SpecialSymbol.java on package element
 */
package element;

import java.util.*;

/**
 * @author carlos
 * 
 * <p>Los objetos de esta clase representan elementos simples de la notacin de los
 * esquemas de anlisis sintctico que no corresponden a smbolos de la
 * gramtica, reglas o valores. Un ejemplo es el punto de las producciones
 * que aparece en esquemas como el del algoritmo de Earley.</p>
 * <p>Al igual que en el caso de los smbolos y las posiciones de cadena
 * de entrada, la igualdad de los SpecialElements se establece por identidad,
 * es decir, todas las ocurrencias de un mismo elemento de la notacin
 * sern representadas internamente por el mismo objeto.</p>
 *
 * Created at regulus, 16-oct-2004 12:53:37
 */
public class SpecialElement extends SimpleElement implements ArrayIndexable , HashIndexable
{
	
	/**
	 * Devuelve siempre 0.
	 */
	public int getArrayIndex()
	{
		return 0;
	}
	
	public Object getHashKey()
	{
		return name;
	}
	
	/**
	 * Devuelve siempre 1. Por defecto, suponemos que los SpecialElement van a ser
	 * Singletons.
	 */
	public static int getArrayLength()
	{
		return 1;
	}
	
	/**Cadena que sirve de nombre al elemento.*/
	private String name;
	
	/**
	 * Tabla hash que relaciona nombres con instancias de
	 * la clase SpecialElement.
	 */
	private static HashMap instances = new HashMap();

	/**
	 * Este mtodo devuelve siempre true para los elementos de esta clase,
	 * dado que se considera que todos los elementos especiales son
	 * constantes (ground).
	 * @see Element#isGround()
	 */
	public boolean isGround() 
	{
		return true;
	}
	
	/**
	 * Devuelve una representacin del elemento en forma de cadena, que es la misma
	 * cadena que se utiliz para llamar a fromString() y obtener la instancia.
	 */
	public String toString()
	{
		return name;
	}
	
	/**
	 * Devuelve una instancia de SpecialElement a partir de una representacin
	 * de la misma en forma de cadena. Este mtodo garantiza que cadenas iguales 
	 * darn como resultado la misma (idntica) instancia.
	 * En esta implementacin, tambin se garantiza que cadenas distintas
	 * darn lugar a instancias distintas; pero esto no forma parte del contrato
	 * del mtodo y podra no cumplirse en las subclases.
	 * @param s Cadena que utilizamos para identificar a un elemento especial.
	 * @return Instancia de la clase SpecialElement correspondiente a la
	 * cadena dada.
	 */
	public static SpecialElement fromString(String s)
	{
		SpecialElement se = (SpecialElement) instances.get(s);
	    if ( se != null )
	    	return se;
	    else
	    {
	        //nueva
	        SpecialElement nueva = new SpecialElement(s);
	        instances.put(s,nueva);
	        return nueva;
	    }
	}
	
	protected void registerInstance(SpecialElement se)
	{
		instances.put(se.name,se);
	}
	
	
	/**
	 * Construye una instancia de SpecialElement asocindole como
	 * nombre la cadena dada como parmetro.
	 * @param s Cadena que identificar al elemento especial.
	 */
	protected SpecialElement ( String s )
	{
		name = s;
	}
	
	/**
	 * Devuelve el cdigo Java para que los elementos especiales puedan
	 * aparecer en expresiones.
	 */
	public String getExpressionCode()
	{
		return "SpecialElement.fromString(\""+name+"\")";
	}

}
