package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;

/**
 * Esta clase representa una expresin booleana de tipo Is-Terminal(A), que se
 * utiliza para comprobar si un smbolo es terminal.
 * @author Carlos
 *
 */
public class IsTerminalExpression extends ExpressionElement 
{

	public IsTerminalExpression(Element argument)
	{
		super( new Element[] { argument } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	 /**
     * La expresin Is-Terminal se resuelve al cdigo Java que llama a un mtodo
     * que determina si un smbolo es terminal.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("grammar.CFGUtils.isTerminal(");
        result.append(memberNames[0]);
        result.append(")");
        return result.toString();
	}
	
	public static IsTerminalExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Is-Terminal expression taking " + expressionArgs.size() + " arguments when 1 is expected");
			}
			
			Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new IsTerminalExpression ( argument );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
