package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.Element;
import element.ExpressionElement;

/**
 * This class represents an expression of the form Not(something), where "something"
 * is a boolean expression.
 * A NotExpression is translates to Java code which negates the enclosed expression.
 * 
 * @author carlos
 * Created at denebola, 13/11/2006 20:48:47
 *
 */
public class NotExpression extends ExpressionElement
{

	public NotExpression(Element argument1 )
	{
		super( new Element[] { argument1 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return element.BooleanElement.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("BooleanElement.groundFromValue(");
        result.append("!(");
        result.append(memberNames[0]);
        result.append(".getValue())");
        result.append(")");
        return result.toString();
	}
	
	public static NotExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Not expression taking " + expressionArgs.size() + " arguments when 1 is expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new NotExpression ( argument1 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
