package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents a boolean expression of the form Is-anchor-of ( symbol ; tree )
 * @author Carlos
 *
 */
public class IsAnchorOfExpression extends ExpressionElement 
{

	public IsAnchorOfExpression(Element argument1 , Element argument2)
	{
		super( new Element[] { argument1 , argument2 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("BooleanElement.groundFromValue(");
        result.append(memberNames[1]);
        result.append(".hasAnchor( ");
        result.append(memberNames[0]);
        result.append(" )");
        result.append(")");
        return result.toString();
	}
	
	public static IsAnchorOfExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 2 )
			{
				throw new ElementParseException("Is Anchor Of expression taking " + expressionArgs.size() + " arguments when 2 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
			
			return new IsAnchorOfExpression ( argument1 , argument2 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
