package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.SumOfIntegersExpression;
import element.Symbol;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Esta clase representa la expresin "Symbol-Of" que, dado un nodo, nos devuelve el smbolo
 * al que est asociado.
 */
public class SymbolOfExpression extends ExpressionElement 
{

	public SymbolOfExpression(Element argument)
	{
		super( new Element[] { argument } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return Symbol.class;
	}
	
	 /**
     * La expresin Symbol-of se resuelve en un cdigo java que llama al mtodo 
     * getAssociatedSymbol() del TAGTreeNode correspondiente.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append(memberNames[0]);
        result.append(".getAssociatedSymbol()");
        return result.toString();
	}
	
	public static SymbolOfExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 1 )
			{
				throw new ElementParseException("Symbol-of expression taking " + expressionArgs.size() + " arguments when 1 are expected");
			}
			
			Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			
			return new SymbolOfExpression ( argument );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
