package element.tag.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.Element;
import element.ExpressionElement;
import element.Symbol;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;
import element.tag.TAGTree;

/**
 * Esta clase representa la expresin "Tree-Of" que, dado un nodo, nos devuelve el rbol
 * de la TAG al que est asociado.
 */
public class TreeOfExpression extends ExpressionElement
{
	
	public TreeOfExpression(Element argument)
	{
		super( new Element[] { argument } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return TAGTree.class;
	}
	
	/**
     * La expresin Tree-of se resuelve en un cdigo java que llama al mtodo 
     * getAssociatedTree() del TAGTreeNode correspondiente.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append(memberNames[0]);
        result.append(".getAssociatedTree()");
        return result.toString();
	}
	
	public static TreeOfExpression fromString ( String sparm ) throws ElementParseException
	{
		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			List expressionArgs = fep.getArguments();
			if ( expressionArgs.size() != 1 )
				throw new ElementParseException("Tree-of expression taking " + expressionArgs.size() + " arguments when 1 are expected");
			Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			return new TreeOfExpression ( argument );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
	}

}
