package grammar;

import element.*;

import java.util.*;
import java.io.*;

/**
 * Gramtica independiente del contexto con mtodo de lectura
 * para fichero de Alvey.
 *  
 * @author carlos
 *
 */
public class PrologFormatContextFreeGrammar extends ContextFreeGrammar
{
		
	/**
	 * Construye y devuelve la gramtica vaca.
	 *
	 */
	public PrologFormatContextFreeGrammar ()
	{
		super();
	}
	
	public void initFromFile ( File f )
	{
		try
		{
			FileInputStream fin = new FileInputStream( f );
			BufferedReader br = new BufferedReader ( new InputStreamReader ( fin ));
			String linea;
			while ( (linea = br.readLine()) != null )
			{
				if ( !linea.startsWith("grammar_rule(") )
					continue;
				linea = linea.substring(13);
				if ( !linea.endsWith(")."))
					continue;
				linea = linea.substring(0,linea.length()-2);
				StringTokenizer st = new StringTokenizer(linea,",[]");
				if ( !st.hasMoreTokens() ) continue;
				st.nextToken();
				if ( !st.hasMoreTokens() ) continue;
				ArrayList rule = new ArrayList();
				String leftSide = st.nextToken();
				Symbol leftSideSymbol = SymbolTable.instance().symbolFor(leftSide.trim());
				rule.add(leftSideSymbol);
				while ( st.hasMoreTokens() )
					rule.add(SymbolTable.instance().symbolFor(st.nextToken().trim()));
				this.add(rule.toArray(new Symbol[0]));
			}
		}
		catch ( Throwable t )
		{
			System.err.println("Error reading grammar:");
			t.printStackTrace();
		}
	}
	
	/**
	 * Construye una gramtica a partir de un fichero en formato Susanne.
	 * @param f Fichero de donde se leer la gramtica (formato Susanne)
	 */
	public PrologFormatContextFreeGrammar ( File f )
	{
		this();	
		initFromFile(f);
	}
	
	/**
	 * Devuelve una representacin humanamente legible de la gramtica.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer("");
		Iterator it = this.iterator();
		while ( it.hasNext() )
		{
			Symbol[] current = (Symbol[]) it.next();
			sb.append(SymbolTable.instance().getName(current[0]));
			sb.append(" -> ");
			for ( int i = 1 ; i < current.length ; i++ )
			{
				sb.append(SymbolTable.instance().getName(current[i]));
				sb.append(" ");
			}
			sb.append("\n");
		}
		return new String(sb);
	}
	

	
	public Element[] getEntity ( int i )
	{
		return (Element[])super.get(i);
	}
	
	
	
}