package agenda;
import java.util.NoSuchElementException;

/**
 * This interface represents an agenda. An agenda is a data structure holding
 * items that have been obtained in a deductive process and have not yet been
 * used to trigger new deductions.
 * The simplest implementation of an agenda is a queue, but other implementations,
 * such as a priority queue, can be interesting for some applications.
 * @author carlos
 *
 * Created at regulus, 11-nov-2006 12:23:59
 */
public interface Agenda
{
	
	/**
	 * Adds an item to the agenda.
	 * @param item The item to add.
	 * @return Always returns true, like Java's Collection.add().
	 */
	public boolean add ( Object item );
	
	/**
	 * Removes and returns the first item of the agenda.
	 * The notion of first item is defined by the implementation.
	 * @return The first item in the agenda, as defined by the concrete agenda implementation.
	 * @throws NoSuchElementException if the agenda is empty.
	 */
	public Object removeFirst ( ) throws NoSuchElementException;
	
	/**
	 * Checks whether the agenda is empty.
	 * @return true if the agenda is empty, false otherwise.
	 */
	public boolean isEmpty();
	
	/**
	 * Returns the number of items in the agenda.
	 */
	public int size();
	
	/**
	 * Empties the agenda, removing all items.
	 */
	public void clear();
	
}
