package element;


/**
 * @author carlos
 *
 * Elementos que usa el generador de cdigo como elementos raz auxiliares
 * en la visin que tiene de los elementos segn el patrn Composite. El sparser pasa al
 * generator un rbol en que los tems aparecen representados como ItemNodes que tienen
 * ElementNodes que tienen Elements; pero para generar cdigo ser interesante pasar esta
 * representacin (al menos en ciertos momentos) a un composite de Element que empieza en
 * un GenericWrapperElement que representa el propio tem.
 */
public class GenericWrapperElement extends CompositeElement
{

	/**Elementos que tiene dentro el wrapper.*/
	private Element[] ar; 
	
	/*Podemos construirlo a partir de un ElementAggregate cualquiera o de un array de Elements.*/
	
	/**
	 * Construye un elemento envoltorio a partir de un array de elementos,
	 * que sern sus hijos.
	 */
	public GenericWrapperElement ( Element[] ar )
	{
		this.ar = ar;
	}
	
	/**
	 * Construye un elemento envoltorio a partir de un agregado de
	 * elementos, que sern sus hijos.
	 * @param ea Agregado de elementos que sern hijos del nuevo
	 * elemento compuesto.
	 */
	public GenericWrapperElement ( ElementAggregate ea )
	{
		ar = new Element[ea.numChildren()];
		for ( int i = 0 ; i < ea.numChildren() ; i++ )
		{
			ar[i] = ea.getChild(i);
		}
	}
	
	/**
	 * Se considera que un elemento de esta clase est instanciado
	 * si todos sus hijos lo estn.
	 * @see element.Element#isGround()
	 */
	public boolean isGround() 
	{
		for ( int i = 0 ; i < ar.length ; i++ )
		{
			if ( !ar[i].isGround() ) return false;
		}
		return true;
	}

	/**
	 * @see util.ElementAggregate#numChildren()
	 */
	public int numChildren() 
	{
		return ar.length;
	}

	/**
	 * @see util.ElementAggregate#getChild(int)
	 */
	public Element getChild(int i) 
	{
		return ar[i];
	}

}
