package element.fs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;

import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

/**
 * Represents an expression of the form Word-tag(word) which gets
 * the given word's associated feature structure from the lexicon.
 * @author Carlos
 *
 */
public class WordTagExpression extends ExpressionElement
{

		
		public WordTagExpression(Element argument)
		{
			super( new Element[] { argument } );
		}
		
		public boolean isGround()
		{
			return false;
		}
		
		public Class getResultingClass()
		{
			return element.Symbol.class;
		}
		
		public String getExpressionCode(String[] memberNames) 
		{
			//TODO !!
			//This is provisional.
			
			StringBuffer sb = new StringBuffer("");
			sb.append("SymbolTable.instance().symbolFor((element.fs.WordFeaturesExpression.getWordInformation(");
			sb.append(memberNames[0]);
			sb.append(".getName())[0]).getTag())");
			
			
			return new String(sb);
		}
		
		public static WordTagExpression fromString ( String sparm ) throws ElementParseException
		{
			try
			{
				FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
				List expressionArgs = fep.getArguments();
				if ( expressionArgs.size() != 1 )
					throw new ElementParseException("Word Tag expression taking " + expressionArgs.size() + " arguments when 1 are expected");
				Element argument = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
				return new WordTagExpression ( argument );
				
			}
			catch ( FunctionExpressionParseException fepe )
			{
				throw new ElementParseException(fepe);
			}
		}

}
