package engine;

import java.util.List;
import schema.item.ItemHandler;
import schema.step.StepHandler;

import agenda.Agenda;

/**
 * Pluggable deductive engine support.
 * 
 * This interface represents a deductive engine, which works with items and an agenda.
 * 
 * @author Carlos Gmez
 *
 */
public interface DeductiveEngine 
{
		
	/**
	 * Generates all the items that can be generated just with the information provided by the grammar,
	 * before the string is known.
	 * Note that in the standard deductive engine, this method's implementation is just a call to process(), 
	 * for the deductive engine always works in the same way, except for the fact that it has less items
	 * to start with in this case. However, in extensions to the deductive engine such as regional error
	 * correction, both methods may be different (for example, preprocess() does not trigger error correcting
	 * steps
	 * @return List of all the items in the ItemHandler.
	 */
	public List preprocess();
	
	
	/**
	 * Generates all the items that can be generated with the information provided by grammar and string.
	 * See preprocess() for the difference between process() and preprocess().
	 * 
	 * @param a
	 * @return
	 */
	public List process ();
	
	
	
	public void setItemHandler ( ItemHandler ih );
	
	
	public Object getProperty ( String key );
	
	
}