package grammar.lcrc;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.predicate.FunctionExpressionParseException;
import element.predicate.FunctionExpressionParser;

public class RightCornerExpression extends ExpressionElement 
{

	public RightCornerExpression(Element argument1 , Element argument2)
	{
		super( new Element[] { argument1 , argument2 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return BooleanElement.class;
	}
	
	 /**
     * La expresin Left-Corner se resuelve al cdigo Java que llama a un mtodo
     * que determina si dos smbolos son o no left-corner.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append("((grammar.lcrc.LCRCContextFreeGrammar.instance)).rightCorner(");
        for ( int i = 0 ; i < memberNames.length ; i++ )
        {
        	if ( i > 0 )
        		result.append(",");
           	result.append("(");
           	result.append(memberNames[i]);
           	result.append(")");
        }
        result.append(")");
        return result.toString();
	}
	
	public static RightCornerExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 2 )
			{
				throw new ElementParseException("Left-Corner expression taking " + expressionArgs.size() + " arguments when 2 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
			
			return new RightCornerExpression ( argument1 , argument2 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
