/*
 * Created at regulus on 16-feb-2005 18:36:19
 * as file FileCopy.java on package util
 */
package util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 * @author carlos
 *
 * Clase utilidad para copiar ficheros de un directorio a otro.
 * Created at regulus, 16-feb-2005 18:36:19
 */
public class FileCopy 
{

	//Carcter de fin de lnea del sistema.
	private static String newLine = System.getProperty("line.separator");
	
	/**
	 * Copia un fichero de un directorio a otro. Ntese que el modo de copiar no es
	 * lo ms eficiente posible; pero es perfectamente aceptable para ficheros pequeos.
	 * Tambin puede copiar directorios, si en lugar de ficheros se le pasan objetos
	 * File representando directorios como argumento. La copia es recursiva, copindose
	 * todo el contenido del subdirectorio. Ntese que, si se produce alguna excepcin
	 * durante la copia, parte del contenido puede haber sido copiado.
	 * @param source Fichero desde el que se va a copiar.
	 * @param destination Fichero al que se va a copiar.
	 * @throws IOException Si no se puede acceder a alguno de los ficheros, no hay
	 * permisos, o sucede cualquier otro problema de entrada/salida.
	 */
	public static void copyFile ( File source , File destination ) throws IOException
	{
		if ( source.isDirectory() )
		{
			//copia recursiva del contenido del directorio
			destination.mkdirs();
			File[] filesInside = source.listFiles();
			for ( int i = 0 ; i < filesInside.length ; i++ )
			{
				File fileToCopy = filesInside[i];
				File newFile = new File(destination,fileToCopy.getName());
				copyFile(fileToCopy,newFile);
			}
		}
		else
		{
			//caso base: copiar un fichero
			FileReader fr = new FileReader(source);
			FileWriter fw = new FileWriter(destination);
			BufferedReader br = new BufferedReader ( fr );
			BufferedWriter bw = new BufferedWriter ( fw );
			
			int len = (int)source.length();
			char[] buff = new char[len];
			int readLen;
			
			//pasamos de un fichero a otro a travs del buffer
			while ( (readLen = br.read(buff,0,len)) > 0 )
				bw.write(buff,0,len);
			
			br.close();
			bw.close();
		}
	}
	

}
