/*
 * Created at regulus on 16-oct-2005 13:05:02
 * as file CFGSentenceGenerator2.java on package auxiliary
 */
package auxiliary;

import element.Element;
import element.Symbol;
import element.SymbolTable;
import grammar.ContextFreeGrammar;
import grammar.PrologFormatContextFreeGrammar;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * @author carlos
 * 
 * This is a random sentence generator for CFG's.
 * It needs the classes related to grammars and symbols.
 *
 * Created at regulus, 16-oct-2005 13:05:02
 */
public class CFGSentenceGenerator2 
{
	
	/**
	 * Stores the grammar rules indexed by the symbol in their left hand side.
	 */
	private Map grammarHash = new HashMap();
	
	/**
	 * The grammar with which we want to generate random sentences.
	 */
	private ContextFreeGrammar cfg = null;
	
	/**
	 * The set of terminals. Unused.
	 */
	private Set terminals = new LinkedHashSet();
	
	/**
	 * The set of nonterminals.
	 */
	private Set nonTerminals = new LinkedHashSet();
	
	
	/**
	 * An unused function.
	 */
	public void loadTerminals ( File f ) throws Throwable
	{
		BufferedReader br = new BufferedReader ( new InputStreamReader ( new FileInputStream( f ) ) );
		String linea;
		while (  (linea=br.readLine())  != null  )
		{
			if ( linea.startsWith("lex(") )
			{
				linea = linea.substring(4);
				StringTokenizer st = new StringTokenizer(linea,",");
				terminals.add( ( SymbolTable.instance().symbolFor(st.nextToken().trim()) ) );
			}
			else
				continue;
		}
	}
	
	
	/**
	 * Stores the grammar rules in a hashtable for efficient access.
	 * Stores the nonterminals in a set.
	 * @param cfg
	 */
	public void initGrammar ( ContextFreeGrammar cfg )
	{
		this.cfg = cfg;
		for ( int i = 0 ; i < cfg.size() ; i++ )
		{
			Element[] rule = cfg.getEntity(i);
			
			List l = (List) grammarHash.get ( rule[0] );
			if ( l == null )
			{
				l = new ArrayList();
				grammarHash.put ( rule[0] , l );
			}
			l.add ( rule );
			int len = rule.length - 1;
			if ( len == 0 ) len0++;
			if ( len == 1 ) len1++;
			if ( len == 2 ) len2++;
			if ( len > 2 ) lenmas++;
			sumlen += len;
			nonTerminals.add( rule[0] );
		}
	}
	
	private int len0 = 0;
	private int len1 = 0;
	private int len2 = 0;
	private int lenmas = 0;
	private long sumlen;
	
	private ArrayList currentSentence;
	private Random randomizer = new Random();
	
	public void initSentence()
	{
		currentSentence = new ArrayList();
		currentSentence.add ( SymbolTable.instance().symbolFor("S") );
	}
	
	public ArrayList getSentence()
	{
		return currentSentence;
	}
	
	public String getSentenceAsString()
	{
		StringBuffer sb = new StringBuffer();
		for ( int i = 0 ; i < currentSentence.size() ; i++ )
		{
			sb.append(currentSentence.get(i).toString());
			sb.append(" ");
		}
		return sb.toString();
	}
	
	/**
	 * Applies one grammar rule to expand the sentence randomly.
	 * @return
	 */
	public boolean expandSentence()
	{
		int expandIndex = Math.abs ( randomizer.nextInt() % currentSentence.size() );
		Symbol expandSymbol = (Symbol) currentSentence.get(expandIndex);
		List rules = (List) grammarHash.get ( expandSymbol );
		if ( rules == null || rules.size() < 1 )
			return false;
		else
		{
			//perform expansion
			int ruleIndex = Math.abs ( randomizer.nextInt() % rules.size() );
			Element[] rule = (Element[]) rules.get(ruleIndex);
			for ( int i = 1 ; i < rule.length ; i++ )
			{
				currentSentence.add(expandIndex+i,rule[i]);
			}
			currentSentence.remove(expandIndex);
			return true;
		}
	}
	
	public static void do1( CFGSentenceGenerator2 generator )
	{
		 generator.initSentence();
		int iters = 0;
		 while ( iters < 200 )
		 {
		 	//System.out.println(generator.getSentence().size());
		 	generator.expandSentence();
		 	iters++;
		 }
		 //System.out.println(generator.getSentence().size() + ": " + generator.getSentenceAsString());
	}
	
	private boolean isSentenceOfTerminals ( List sentence )
	{
		for ( int i = 0 ; i < sentence.size() ; i++ )
		{
			if ( nonTerminals.contains( sentence.get(i) ) )
				return false;
		}
		return true;
	}
	
	public static void do2( CFGSentenceGenerator2 generator )
	{
		 generator.initSentence();
		int iters = 0;
		 while ( iters < 2000 && !generator.isSentenceOfTerminals( generator.getSentence() ) )
		 {
		 	//System.out.println(generator.getSentence().size());
		 	generator.expandSentence();
		 	iters++;
		 }
		 //System.out.println(generator.getSentence().size() + ": " + generator.getSentenceAsString());
	}

	public static void main(String[] args) throws Throwable
	{
		 //ContextFreeGrammar cfg = new ContextFreeGrammar();
		 //cfg.initFromFile(new File("C:\\GeneratedParsers\\SusanneGrammar"));
		
		
		ContextFreeGrammar cfg = new PrologFormatContextFreeGrammar();
		cfg.initFromFile(new File("C:/GeneratedParsers/delft.gra"));
		

		/*
		ContextFreeGrammar cfg = new ContextFreeGrammar();
		cfg.initFromFile(new File("C:/GeneratedParsers/SusanneGrammar"));
		*/
		
		 CFGSentenceGenerator2 generator = new CFGSentenceGenerator2();
		 generator.initGrammar(cfg);
		 //generator.loadTerminals(new File("C:\\GeneratedParsers\\alvey.gra"));
		 
		 int nSymbols = SymbolTable.instance().getLastId()+1;
		 int nNonterminals = generator.nonTerminals.size();
		 
		System.out.println("==Grammar Stats=="); 
		System.out.println("Symbols: " + nSymbols); 
		System.out.println("Nonterminals: " + nNonterminals);
		System.out.println("Terminals: " + (nSymbols-nNonterminals));
		System.out.println("Rules: " + cfg.size());
		System.out.println("Len 0: " + generator.len0);
		System.out.println("Len 1: " + generator.len1);
		System.out.println("Len 2: " + generator.len2);
		System.out.println("Len +: " + generator.lenmas);
		System.out.println("Avg len: " + (double)generator.sumlen / (double)cfg.size());
		System.out.println("==Process Start==");

		//for ( int batchIter = 0 ; batchIter < 100 ; batchIter++ )
		//{

		//	System.err.println("Batch Iteration " + batchIter);


		 List[] sentences = new List[20000];
		 for ( int i = 0 ; i < sentences.length ; i++ )
		 	sentences[i] = new ArrayList();
		 
		 for ( int i = 0 ; i < 500000 ; i++ )
		 {
		 	do2(generator);
			if ( generator.isSentenceOfTerminals(generator.getSentence())
				&& sentences[generator.getSentence().size()].size() < 100 )
				 
		 		sentences[generator.getSentence().size()].add(generator.getSentence());

			//else
			//	System.out.println("FALSE!!");

		 }	
		 
		 for ( int i = 0 ; i < sentences.length ; i++ )
		 {
		 	for ( int j = 0 ; j < sentences[i].size() ; j++ )
		 	{
		 		List sentence = (List) sentences[i].get(j);
		 		StringBuffer sb = new StringBuffer();
				for ( int k = 0 ; k < sentence.size() ; k++ )
				{
					sb.append(sentence.get(k).toString());
					sb.append(" ");
				}
				System.out.println(generator.isSentenceOfTerminals(sentence)+","+sentence.size()+": " + sb.toString());
		 	}
		 }


		//}
		 
	}
	
	
	
	
	
	
	
	
}
