
package element;

/**
 * <p>Interfaz que cumplirn aquellos elementos que se puedan indexar en un array, es
 * decir, las clases de elementos E para las cuales exista una funcin f:E->0..n.
 * Dicha funcin f es la funcin getArrayIndex, y la cota n viene dada por la funcin
 * getArrayLength. Ntese que esta ltima funcin es static y no forma parte de la
 * interfaz, pues no se permiten mtodos static en interfaces.</p>
 * <p>Created 12-ago-2004 12:49:03</p>
 * @author carlos
 */
public interface ArrayIndexable
{

    /**
     * <p>Devuelve un nmero entre 0 y this.getArrayLength()-1 asociado a este elemento.
     * En principio, la nica condicin adicional que debe cumplir necesariamente este mtodo
     * es que sea una funcin (para el mismo elemento, o elementos que sean iguales
     * segn el mtodo equals, debe devolver siempre el mismo valor). Sin embargo,
     * dado que el nmero devuelto se utilizar para indexar items en arrays segn el
     * valor para un elemento, tambin es deseable que los diferentes elementos posibles
     * cubran todo el rango y que la obtencin de este nmero sea muy rpida para que lo
     * sea la bsqueda en el ndice.</p>
     * @return ndice entre 0 y this.getArrayLength()-1.
     */
    public int getArrayIndex();
    
    /**
     * <p>Devuelve la longitud que debe tener un array para abarcar todos los posibles
     * valores de getArrayIndex() que pueden devolver los objetos de esta clase.</p>
     * <p>Ntese que el valor devuelto por este mtodo debe estar definido slo despus
     * de conocida la gramtica, instanciados los pasos y conocida la cadena de entrada; 
     * pero no es necesario que lo est antes. Esto cobra relevancia, por ejemplo, en la
     * clase Symbol: los smbolos son ArrayIndexables porque una vez que se tiene una 
     * gramtica se conocen todos los smbolos y no se van a crear ms, si se llamara a 
     * esta funcin sobre un smbolo antes de haber terminado de instanciar la gramtica 
     * el valor devuelto sera incorrecto. En una clase como StringPosition la cota
     * del array an tarda ms en conocerse, pues necesita la cadena de entrada.</p>
     * @return Longitud que debe tener el array ndice para esta clase de elementos.
     */
    /*
    public static int getArrayLength();
    */
    
    //Adicionalmente al mtodo getArrayIndex(), las clases que implementen ArrayIndexable
    //deben tener el mtodo de clase (static) getArrayLength(), que har lo que dice
    //el javadoc del mtodo comentado de arriba.
    
}
