package element;


/**
 * 
 * @author carlos
 *
 * Elemento simple que representa una cadena de informacin.
 * Dichas cadenas se guardan en tablas hash.
 */
public class InfoTextElement extends SimpleElement
{
	
	/**
	 * Valor de esta cadena, para elementos ground.
	 */
    protected String value;
	
	/**
	 * Atributo que nos dice si la cadena es constante/instanciada (ground) o no.
	 */
	protected boolean ground;
	

	/**
	 * Crea un texto de informacin con el valor dado.
	 * @param value Valor que tomar el texto, si es ground.
	 * @param ground Si el texto es una constante (ground) o no.
	 */
	public InfoTextElement ( String value , boolean ground )
	{
		this.value = value; this.ground = ground;
	}
	
	private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
	private static java.util.HashMap groundHash = new java.util.HashMap(); //keys: String
	
	/**
	 * Mtodo de parsing para InfoTextElements, nos da una cadena de informacin no 
	 * ground para una cadena dada que aparece en el esquema. Llamadas con la misma 
	 * cadena devuelven el mismo objeto.
	 * @param s Cadena que representa la cadena en un esquema, p.ej. "a"
	 * @return Elemento no ground representando la cadena de texto dada.
	 */
	public static InfoTextElement groundInfoTextElement(String s)
	{
		if ( s.charAt(0) == '\"' && s.charAt(s.length()-1) == '\"')
		{
			s = s.substring(1,s.length()-1);
			return groundFromValue(s);
		}
		else { return null; }
	}
	
	/**
	 * Obtiene un InfoTextElement constante a partir de su cadena valor.
	 * Cadenas iguales siempre devolvern el mismo elemento.
	 * @param s Valor de la cadena deseada.
	 * @return InfoTextElement para esa cadena.
	 */
	public static InfoTextElement groundFromValue(String s)
	{
		InfoTextElement ite = (InfoTextElement) groundHash.get(s);
		if ( ite != null ) return ite;
		else
		{
			ite = new InfoTextElement(s,true);
			groundHash.put(s,ite);
			return ite;
		}
	}
	
	/**
	 * Devuelve un InfoTextElement no ground a partir de su representacin
	 * en forma de cadena en el esquema.
	 * @param valString Representacin de un InfoText, por ejemplo text1
	 * @return Una instancia de InfoTextElement con el valor dado, que puede ser nueva
	 * u otra devuelta anteriormente.
	 */
	public static InfoTextElement nonGroundInfoTextElement ( String s )
	{
		if ( nonGroundHash.get(s) != null )
	        return (InfoTextElement) nonGroundHash.get(s);
	    else
	    {
	        InfoTextElement nueva = new InfoTextElement(s,false);
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
		
	/**
	 * Nos dice si este texto es ground (constante) o no.
	 */
	public boolean isGround()
	{
		return ground;
	}
	
	/**
	 * Devuelve el valor de este texto.
	 */
	public final String getValue()
	{
		return value;
	}
		
	/**
	 * Devuelve el cdigo Java para que las cadenas ground aparezcan en expresiones.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	    	//return "InfoTextElement.groundInfoTextElement("+value+")";
	    	return "InfoTextElement.groundFromValue(" + '\"' + value + '\"' + ")";
	    else
	        return null;
	}
	
	/**
	 * Devuelve una representacin de esta cadena de texto.
	 */
	public String toString()
	{
	    if ( isGround() )
	        return value;
	    else
	        return "(ng)"+value;
	}
	
		
	/**
	 * Comparacin de igualdad de cadenas. Estn canonicalizadas, nos vale la identidad.
	 */
	public boolean equals ( Object o )
	{
		return this == o;
	}
	
	
	
	
	
}