package element.fs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;

import element.HashIndexable;
import element.SimpleElement;


/**
 * Correction to the following warning: I think it is now conmutative and OK.
 * 
 * Important warning: current implementation of unify() is not commutative.
 * If we call fs1.unify(fs2), variables at fs1 are kept, but variables at
 * fs2 are not kept (and aren't in fact taken into account for unification).
 * 
 * So if we want to unify for example Mtop Mbot Ptop and Pbot in context of
 * tree beta, we'll do something like
 * 
 * PFS vars = <variables of tree beta>
 * cur = vars.unify(Mtop)
 * cur = vars.unify(Mbot)
 * cur = vars.unify(Ptop)
 * cur = vars.unify(Pbot)
 * to use the variables until the end (comprobations with unifies() have been
 * omitted in this example)
 */


public class OldPlainFeatureStructure extends SimpleElement implements HashIndexable
{

	//test method
	public static void main ( String[] args )
	{
		OldPlainFeatureStructure pfs1 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs2 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs3 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs4 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs5 = new OldPlainFeatureStructure(true);
		pfs1.setFeatureValue("agr","sing");
		pfs2.setFeatureValue("agr","plur");
		System.out.println(pfs1.unifies(pfs2));
		System.out.println(pfs1.unifies(pfs3));
		pfs3.linkFeatureToVariable("agr","X1");
		pfs3.linkFeatureToVariable("agrCopy","X1");
		System.out.println(pfs1);
		System.out.println(pfs3);
		System.out.println(pfs3.unifies(pfs1));
		System.out.println(pfs3.unify(pfs1));
		System.out.println(pfs1.unify(pfs3));
		OldPlainFeatureStructure pfs6 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs7 = new OldPlainFeatureStructure(true);
		OldPlainFeatureStructure pfs8 = new OldPlainFeatureStructure(true);
		pfs6.setFeatureValue("numbers","1/2/3/4/5/6");
		pfs7.setFeatureValue("numbers","1/3/5/7/9/11");
		pfs7.linkFeatureToVariable("numbers","X1");
		System.out.println(pfs6.unifies(pfs7));
		System.out.println(pfs6.unify(pfs7));
		pfs8.linkFeatureToVariable("numeros","X1");
		System.out.println("6:"+pfs6);
		System.out.println("7:"+pfs7);
		System.out.println("8:"+pfs8);
		System.out.println("7 u 6:"+pfs7.unify(pfs6));
		System.out.println("7 u 6:"+unify(pfs7,pfs6));
		System.out.println("8 u (7 u 6):"+pfs8.unify(pfs7.unify(pfs6)));
		System.out.println("(7 u 6) u 8:"+pfs7.unify(pfs6).unify(pfs8));
		System.out.println("(7 u 6) u 8:"+unify(unify(pfs7,pfs6),pfs8));
		System.out.println("Equality: " +pfs8.unify(pfs7.unify(pfs6)).equals(pfs7.unify(pfs6).unify(pfs8)));
	}
	
	/*
	public int hashCode()
	{
		return 37*this.featureToValueMap.size() + this.featureToVarNameMap.size();
	}
	*/
	public int hashCode()
	{
		int hash = 1;
		hash = hash * 31 + ((this.featureToValueMap == null)? 0 : this.featureToValueMap.hashCode());
		hash = hash * 31 + ((this.featureToVarNameMap == null)? 0 : this.featureToVarNameMap.hashCode());
		return hash;
	}
	
	
	public Object getHashKey()
	{
		return this;
	}
	
	public boolean equals ( Object o )
	{
		
		if ( ! ( o instanceof OldPlainFeatureStructure ) ) return false;
		if ( !ground ) return ( o != null && this == o );		
		OldPlainFeatureStructure another = (OldPlainFeatureStructure)o;
		
		/*
		return this.featureToValueMap.equals(another.featureToValueMap)
		&& this.featureToVarNameMap.equals(another.featureToVarNameMap);
		*/
		
		/*
		OPTIMIZED EQUALS order of operations based on a test run with the following data:
		
		total 30,000,000 equals executed
		
		this.featureToValueMap.size() == another.featureToValueMap.size():
		fails 20,180,526 times
		
		this.featureToVarNameMap.size() == another.featureToVarNameMap.size():
		fails 4,361,175 times 
		
		this.featureToValueMap.equals(another.featureToValueMap):
		fails 21,606,377 times
		
		this.featureToVarNameMap.equals(another.featureToVarNameMap):
		fails 5,928,652 times
		
		(of a total of 22,653,834 false results, the other 7,346,166 were true)
		*/
		
		if ( this.featureToValueMap.size() != another.featureToValueMap.size() )
			return false;
		if ( this.featureToVarNameMap.size() != another.featureToVarNameMap.size() )
			return false;
		return this.featureToValueMap.equals(another.featureToValueMap)
		&& this.featureToVarNameMap.equals(another.featureToVarNameMap);
		
		
		//(varNameToFeatureMap is redundant data)
		
		
		/*
		
		Iterator it1 = this.features();
		while ( it1.hasNext() )
		{
			String feat = (String) it1.next();
			Object val = this.getFeatureValue(feat);
			Object val2 = another.getFeatureValue(feat);
			if ( val==null && val2 != null ) return false;
			if ( val2==null && val != null ) return false;
			if ( val!=null && !(val.equals(val2)) ) return false;
			String varName1 = this.getFeatureVariable(feat);
			String varName2 = another.getFeatureVariable(feat);
			if ( varName1==null && varName2 != null ) return false;
			if ( varName2==null && varName1 != null ) return false;
			if ( varName1!=null && !(varName1.equals(varName2)) ) return false;
		}
		Iterator it2 = another.features();
		while ( it2.hasNext() )
		{
			String feat = (String) it2.next();
			Object val = this.getFeatureValue(feat);
			Object val2 = another.getFeatureValue(feat);
			if ( val==null && val2 != null ) return false;
			if ( val2==null && val != null ) return false;
			if ( val!=null && !(val.equals(val2)) ) return false;
			String varName1 = this.getFeatureVariable(feat);
			String varName2 = another.getFeatureVariable(feat);
			if ( varName1==null && varName2 != null ) return false;
			if ( varName2==null && varName1 != null ) return false;
			if ( varName1!=null && !(varName1.equals(varName2)) ) return false;
		}
		return true;
		*/
	}
	
	public String toString()
	{
		if ( ground )
		{
			StringBuffer sb = new StringBuffer();
			Iterator it = this.features();
			sb.append("[");
			while ( it.hasNext() )
			{
				String feature = (String) it.next();
				sb.append(feature);
				sb.append(" = ");
				String val = (String) featureToValueMap.get(feature);
				if ( val != null )
					sb.append(val);
				String var = (String) featureToVarNameMap.get(feature);
				if ( var != null )
				{
					sb.append("<");
					sb.append(var);
					sb.append(">");
				}
				if ( it.hasNext() )
					sb.append(" | ");
			}
			sb.append("]");
			return new String(sb);
		}
		else
		{
			return "ngfs" + super.hashCode();
		}
	}
	
	private Map featureToVarNameMap = new TreeMap();
	private Map featureToValueMap = new TreeMap();
	private Map varNameToFeatureMap = new HashMap(); //of List
	
	
	private boolean ground;
	
	private static java.util.HashMap nonGroundHash = new java.util.HashMap(); //keys: String
	

	public boolean isGround()
	{
		return ground;
	}
	
	/*
	private static int lastNonGroundInstance = -1;
	private int nonGroundInstanceNumber = 0;
	*/
	
	public static OldPlainFeatureStructure nonGroundFromString(String s)
	{
	    if ( nonGroundHash.get(s) != null )
	    {
	    	OldPlainFeatureStructure pfs = (OldPlainFeatureStructure) nonGroundHash.get(s);
	    	return pfs;
	    }
	    else
	    {
	        //nueva
	        OldPlainFeatureStructure nueva = new OldPlainFeatureStructure(false);
	        /*
	        lastNonGroundInstance++;
	        nueva.nonGroundInstanceNumber = lastNonGroundInstance;
	        */
	        nonGroundHash.put(s,nueva);
	        return nueva;
	    }
	}
	
	private static OldPlainFeatureStructure emptyInstance = new OldPlainFeatureStructure ( true );
	
	public static OldPlainFeatureStructure getEmptyInstance()
	{
		return emptyInstance;
	}
	
	public static OldPlainFeatureStructure emptyFromString ( String s )
	{
		return emptyInstance;
	}
	
	/**
	 * Devuelve el cdigo Java para que las estructuras de rasgos ground
	 * aparezcan en expresiones. Slo soporta la vaca.
	 */
	public String getExpressionCode()
	{
	    if ( ground )
	    {
	    	if ( !featureToVarNameMap.isEmpty() || ! featureToValueMap.isEmpty() || !varNameToFeatureMap.isEmpty() )
	    	{
	    		System.err.println("[ERROR] The empty feature structure is the only FS supported by PlainFeatureStructure.getExpressionCode()");
	    	}
	    	return "element.fs.PlainFeatureStructure.getEmptyInstance()";
	    }
	    else
	        return null;
	}
	
	public OldPlainFeatureStructure ( boolean ground ) 
	{
		this.ground = ground;
	}
	
	/*
	public PlainFeatureStructure ( PlainFeatureStructure toCloneFrom )
	{
		this.ground = toCloneFrom.ground;
		Iterator it = toCloneFrom.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			Object varNameEntry = toCloneFrom.featureToVarNameMap.get(feature);
			Object valueEntry = toCloneFrom.featureToValueMap.get(feature);
			if ( varNameEntry != null )
				this.featureToVarNameMap.put( feature , varNameEntry ); //deep copy
			if ( valueEntry != null )
				this.featureToValueMap.put( feature , valueEntry ); //deep copy
			if ( varNameEntry != null )
			{
				if ( this.varNameToFeatureMap.get(varNameEntry) == null )
					this.varNameToFeatureMap.put(varNameEntry,new ArrayList());
				List featList = (List) this.varNameToFeatureMap.get(varNameEntry);
				featList.add(feature);
			}
		}
	}
	*/
	
	public OldPlainFeatureStructure ( OldPlainFeatureStructure toCloneFrom )
	{
		//newer, faster implementation of the copy-constructor
		this.ground = toCloneFrom.ground;
		
		//a shallow copy is OK in these for they only store strings.
		this.featureToValueMap = (Map) ((TreeMap)toCloneFrom.featureToValueMap).clone();
		this.featureToVarNameMap = (Map) ((TreeMap)toCloneFrom.featureToVarNameMap).clone();
		
		//however, it's not OK for the map which stores lists of strings.
		//in this case we shallow-copy the lists, not the map.
		Iterator it = toCloneFrom.varNameToFeatureMap.keySet().iterator();
		while ( it.hasNext() )
		{
			String varName = (String) it.next();
			ArrayList list = (ArrayList) toCloneFrom.varNameToFeatureMap.get(varName);
			Object l2 = list.clone();
			this.varNameToFeatureMap.put(varName,l2);
		}
		
	}
	
	public Iterator features()
	{
		Set one = featureToValueMap.keySet();
		Set two = featureToVarNameMap.keySet();
		Set union = new TreeSet();
		union.addAll(one);
		union.addAll(two);
		return union.iterator();
		/*
		Iterator itTwo = two.iterator();
		while ( itTwo.hasNext() ) one.add ( itTwo.next() );
		return one.iterator();
		*/
	}
	
	public boolean isFeatureLinked ( String feature )
	{
		return featureToVarNameMap.get(feature) != null;
	}
	
	public boolean isFeatureDefined ( String feature )
	{
		return featureToValueMap.get(feature) != null;
	}
	
	public boolean isVariableDefined ( String variable )
	{
		List feats = (List) varNameToFeatureMap.get(variable);
		if ( feats == null || feats.size() < 1 ) //variable doesn't exist
			return false;
		else
		{
			String aLinkedFeature = (String) feats.get(0);
			return isFeatureDefined ( aLinkedFeature );
		}
	}
	
	public String getFeatureValue ( String feature )
	{
		return (String) featureToValueMap.get(feature);
	}
	
	public String getFeatureVariable ( String feature )
	{
		return (String) featureToVarNameMap.get(feature);
	}
	
	public String getVariableValue ( String variable )
	{
		List feats = (List) varNameToFeatureMap.get(variable);
		if ( feats == null || feats.size() < 1 ) //variable doesn't exist
			return null;
		else
		{
			String aLinkedFeature = (String) feats.get(0);
			return getFeatureValue(aLinkedFeature);
		}
	}
	
	/**
	 * Usage example: given two strings "a/b/d/e/f/h" and "b/z/g/e/f",
	 * this would return a string "b/e/f"
	 */
	private static String disjunctionIntersection ( String s1 , String s2 )
	{
		String sep = "/";
		if ( s1.indexOf(sep) < 0 && s2.indexOf(sep) < 0 && !s1.equals(s2) ) return "";
		Set oneSet = new TreeSet();
		StringTokenizer st = new StringTokenizer(s1,sep);
		while ( st.hasMoreTokens() ) oneSet.add(st.nextToken());
		StringBuffer result = new StringBuffer();
		StringTokenizer st2 = new StringTokenizer(s2,sep);
		boolean nonEmptyResult = false;
		while ( st2.hasMoreTokens() )
		{
			String tok = st2.nextToken();
			if ( oneSet.contains(tok) )
			{
				if ( nonEmptyResult )
					result.append(sep);
				nonEmptyResult = true;
				result.append(tok);
			}	
		}
		return new String(result);
	}
	
	//true = success, false = not done (colliding values)
	public boolean linkFeatureToVariable ( String feature , String varName )
	{
		String varValue = getVariableValue ( varName );
		String featValue = getFeatureValue ( feature );
		if ( featValue != null && varValue != null && !featValue.equals(varValue) )
		{
			//feature's current value doesn't exactly match variable's current value.
			//but they may be intersecting disjunctions.
			String intersect = disjunctionIntersection ( featValue , varValue );
			if ( intersect.length() < 1 )
				return false; //values don't match
			else
			{
				varValue = intersect;
				featValue = intersect;
				//... and set the values!
			}
		}
		featureToVarNameMap.put(feature,varName);
		List featList = (List) varNameToFeatureMap.get(varName);
		if ( featList == null ) varNameToFeatureMap.put(varName,new ArrayList());
		featList = (List) varNameToFeatureMap.get(varName);
		featList.add ( feature );
		if ( featValue == null && varValue != null )
			featureToValueMap.put(feature,varValue);
		if ( featValue != null )
		{
			List feats = (List) varNameToFeatureMap.get(varName);
			if ( feats != null )
			{
				for ( int i = 0 ; i < feats.size() ; i++ )
				{
					String aLinkedFeature = (String) feats.get(i);
					featureToValueMap.put(aLinkedFeature,featValue);
				}
			}
		}
		return true;
	}
	
	public boolean setVariableValue ( String varName , String value )
	{
		String currentValue = (String) getVariableValue ( varName );
		if ( currentValue != null && currentValue != value ) 
		{
			String intersect = disjunctionIntersection(currentValue,value);
			if ( intersect.length() < 1 )
				return false;
			else
				value = intersect; //and set the value
		}
		List feats = (List) varNameToFeatureMap.get(varName);
		if ( feats != null )
		{
			for ( int i = 0 ; i < feats.size() ; i++ )
			{
				String aLinkedFeature = (String) feats.get(i);
				featureToValueMap.put(aLinkedFeature,value);
			}
		}
		return true;
	}
	
	//return true if feature didn't already have a different value
	//(i.e. if a value can be successfully set)
	public boolean setFeatureValue ( String feature , String value )
	{
		String currentValue = (String) featureToValueMap.get(feature);
		if ( currentValue != null && !currentValue.equals(value) ) 
		{
			String intersect = disjunctionIntersection(currentValue,value);
			if ( intersect.length() < 1 )
				return false;
			else
				value = intersect; //and set the value
		}
		String currentVarName = (String)featureToVarNameMap.get(feature);
		List feats = (List) varNameToFeatureMap.get(currentVarName);
		if ( feats != null )
		{
			for ( int i  = 0 ; i < feats.size() ; i++ )
			{
				String aLinkedFeature = (String) feats.get(i);
				if ( getFeatureValue(aLinkedFeature) != null && !getFeatureValue(aLinkedFeature).equals(value) && disjunctionIntersection(getFeatureValue(aLinkedFeature),value).length()<1 )
					return false;
			}
		}
		featureToValueMap.put(feature,value);
		if ( feats != null )
		{
			for ( int i  = 0 ; i < feats.size() ; i++ )
			{
				String aLinkedFeature = (String) feats.get(i);
				featureToValueMap.put(aLinkedFeature,value);
			}
		}
		return true;
	}
	
	/*
	public boolean unifies ( PlainFeatureStructure anotherFS )
	{		
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			if ( this.isFeatureDefined(feature) && 
					this.getFeatureValue(feature) != anotherFS.getFeatureValue(feature) )
				return false;
		}
		return true;	
	}
	*/
	
	public boolean unifies ( OldPlainFeatureStructure anotherFS )
	{
		if ( anotherFS == emptyInstance ) return true; //shortcut for empty fs
		OldPlainFeatureStructure result = new OldPlainFeatureStructure(this);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			//OLD
			/*
			String feature = (String) it.next();
			if ( result.isFeatureDefined(feature) && 
					result.getFeatureValue(feature) != anotherFS.getFeatureValue(feature) &&
					disjunctionIntersection(result.getFeatureValue(feature),anotherFS.getFeatureValue(feature)).length() < 1 )
				return false;
			else
				result.setFeatureValue( feature , anotherFS.getFeatureValue(feature) ); //already sets var if present		
			*/
			String feature = (String) it.next();
			if ( anotherFS.isFeatureDefined(feature) )
			{
				if ( !result.setFeatureValue(feature,anotherFS.getFeatureValue(feature)) )
					return false;
			}
			if ( anotherFS.isFeatureLinked(feature) )
			{
				if ( !result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature)) )
					return false;
			}
		}
		return true;
	}
	
	//PRE: unifies() returns true
	//Note that unification treats feature structures as immutable objects.
	//I.e. once all feature structures have been initted with the setter
	//methods, they can be considered immutable to all effects.
	public OldPlainFeatureStructure unify ( OldPlainFeatureStructure anotherFS )
	{
		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
		OldPlainFeatureStructure result = new OldPlainFeatureStructure(this);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			
			//OLD
			/*
			if ( anotherFS.isFeatureDefined(feature) )
				result.setFeatureValue( feature , anotherFS.getFeatureValue(feature) ); //already sets var if present
			*/

			if ( anotherFS.isFeatureDefined(feature) )
				result.setFeatureValue(feature,anotherFS.getFeatureValue(feature));
			if ( anotherFS.isFeatureLinked(feature) )
				result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature));
			
		}
		return result;
	}
	
	public OldPlainFeatureStructure unifyKeepingVariablesOnly ( OldPlainFeatureStructure anotherFS )
	{
		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
		OldPlainFeatureStructure result = this.unify(anotherFS);
		Iterator it = result.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			String varName = result.getFeatureVariable ( feature );
			if ( varName == null )
				result.featureToValueMap.remove(feature);
		}
		return result;
	}
	
	public OldPlainFeatureStructure unifyKeepingMyVariablesOnly ( OldPlainFeatureStructure anotherFS )
	{
		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
		OldPlainFeatureStructure result = this.unify(anotherFS);
		Iterator it = this.features();
		Set validFeatSet = new LinkedHashSet();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			validFeatSet.add(feature);
			String varName = result.getFeatureVariable ( feature );
			if ( varName == null )
				result.featureToValueMap.remove(feature);
		}
		Iterator it2 = result.features();
		while ( it2.hasNext() )
		{
			String feature = (String) it2.next();
			if ( !validFeatSet.contains(feature) )
			{
				result.featureToValueMap.remove(feature);
				result.featureToVarNameMap.remove(feature);
			}
		}
		return result;
	}
	
	//don't unify anything, just return the result of
	//deleting from anotherFS all variables
	//not appearing in this FS.
	public OldPlainFeatureStructure keepVariables ( OldPlainFeatureStructure anotherFS )
	{
		if ( anotherFS == emptyInstance ) return this; //shortcut for empty fs
		OldPlainFeatureStructure result = new OldPlainFeatureStructure(anotherFS);
		Set s = this.varNameToFeatureMap.keySet();
		Iterator it = result.varNameToFeatureMap.keySet().iterator();
		List l = new ArrayList();
		while ( it.hasNext() ) { l.add(it.next()); }
		it = l.iterator();
		while ( it.hasNext() )
		{
			String varName = (String) it.next();
			if ( !s.contains(varName) )
			{	
				List linkedFeats = (List) result.varNameToFeatureMap.get(varName);
				for ( int i = 0 ; i < linkedFeats.size() ; i++ )
				{
					String linkedFeat = (String) linkedFeats.get(i);
					result.featureToVarNameMap.remove(linkedFeat);
				}
				result.varNameToFeatureMap.remove(varName);
			}
		}
		return result;
	}
	
	
	//PRE: unifies() returns true
	/*
	public PlainFeatureStructure unifyKeepingMyVariablesOnly ( PlainFeatureStructure anotherFS )
	{
		PlainFeatureStructure result = new PlainFeatureStructure(this);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			FSVarOrValue vov = this.getMapEntry(feature);
			if ( vov != null && vov instanceof FSVariable )
			{
				FSValue newVal = new FSValue();
				newVal.setTo(anotherFS.getValue(feature));
				((FSVariable)vov).setValue(newVal);
			}
		}
		return result;
	}
	*/
	
	
	
	/**
	 * NEW, FASTER UNIFICATION IMPLEMENTATION STARTS BELOW
	 * - Results are stored in a variable so that tentative unification can
	 * be performed in side conditions, and the result item can include the
	 * unification result without calculating unification again.
	 * - Unifications are implemented as static methods so that nulls can
	 * be easily handled.
	 */
	
	private static OldPlainFeatureStructure unificationResult;
	
	public static OldPlainFeatureStructure doUnify ( OldPlainFeatureStructure oneFS , OldPlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs
		OldPlainFeatureStructure result = new OldPlainFeatureStructure(oneFS);
		Iterator it = anotherFS.features();
		while ( it.hasNext() )
		{
			String feature = (String) it.next();
			
			//OLD
			/*
			if ( anotherFS.isFeatureDefined(feature) )
				result.setFeatureValue( feature , anotherFS.getFeatureValue(feature) ); //already sets var if present
			*/

			if ( anotherFS.isFeatureDefined(feature) )
			{
				if ( !result.setFeatureValue(feature,anotherFS.getFeatureValue(feature)) )
				{
					return null;
				}
			}
			if ( anotherFS.isFeatureLinked(feature) )
			{
				if ( !result.linkFeatureToVariable(feature,anotherFS.getFeatureVariable(feature)) )
				{
					return null;
				}
			}
		}
		return result;
	}
	
	public static OldPlainFeatureStructure doKeepVariables ( OldPlainFeatureStructure oneFS , OldPlainFeatureStructure anotherFS )
	{
		if ( oneFS == null || anotherFS == null ) return null;
		if ( anotherFS == emptyInstance ) return oneFS; //shortcut for empty fs
		//if ( oneFS == emptyInstance ) return anotherFS; //shortcut for empty fs //not here, this doesn't eliminate vars!
		OldPlainFeatureStructure result = new OldPlainFeatureStructure(anotherFS);
		Set s = oneFS.varNameToFeatureMap.keySet();
		Iterator it = result.varNameToFeatureMap.keySet().iterator();
		List l = new ArrayList();
		while ( it.hasNext() ) { l.add(it.next()); }
		it = l.iterator();
		while ( it.hasNext() )
		{
			String varName = (String) it.next();
			if ( !s.contains(varName) )
			{	
				List linkedFeats = (List) result.varNameToFeatureMap.get(varName);
				for ( int i = 0 ; i < linkedFeats.size() ; i++ )
				{
					String linkedFeat = (String) linkedFeats.get(i);
					result.featureToVarNameMap.remove(linkedFeat);
					
					//TODO Determinar por qu aqu muchas veces llegan FS's con
					//entradas con el valor a null. Se supone que una entrada
					//nunca debera llegar a tener valor null, en todo caso
					//se quitara.
					if ( result.featureToValueMap.get(linkedFeat) == null )
						result.featureToValueMap.remove(linkedFeat);
					
				}
				result.varNameToFeatureMap.remove(varName);
			}
		}
		return result;
	}
	
	public static OldPlainFeatureStructure unify ( OldPlainFeatureStructure oneFS , OldPlainFeatureStructure anotherFS )
	{
		OldPlainFeatureStructure unifResult = doUnify ( oneFS , anotherFS );
		unificationResult = unifResult;
		return unifResult;
	}
	
	public static OldPlainFeatureStructure keepVariables ( OldPlainFeatureStructure oneFS , OldPlainFeatureStructure anotherFS )
	{
		OldPlainFeatureStructure unifResult = doKeepVariables ( oneFS , anotherFS );
		unificationResult = unifResult;
		return unifResult;
	}
	
	public static boolean unifies ( OldPlainFeatureStructure oneFS , OldPlainFeatureStructure anotherFS )
	{
		OldPlainFeatureStructure unifResult = doUnify ( oneFS , anotherFS );
		unificationResult = unifResult;
		return (unificationResult != null);
	}
	
	public static OldPlainFeatureStructure getUnificationResult ()
	{
		return unificationResult;
	}
	
	
}
