package element.predicate;

import java.util.List;

import parser.eparser.ElementParseException;
import parser.eparser.ElementParser;
import element.BooleanElement;
import element.Element;
import element.ExpressionElement;
import element.NullableStringPosition;
import element.SumOfIntegersExpression;

/**
 * Represents a boolean expression of the form union(p1;p2) where p1 and p2
 * are nullable string positions.
 */
public class PositionUnionExpression extends ExpressionElement 
{

	public PositionUnionExpression(Element argument1 , Element argument2)
	{
		super( new Element[] { argument1 , argument2 } );
	}
	
	public boolean isGround()
	{
		return false;
	}
	
	public Class getResultingClass()
	{
		return element.NullableStringPosition.class;
	}
	
	 /**
     * La expresin unin se resuelve al cdigo Java que hace la unin de dos
     * NullableStringPosition.
     */
	public String getExpressionCode(String[] memberNames) 
	{
        StringBuffer result = new StringBuffer("");
        result.append(memberNames[0]);
        result.append(".union(");
        result.append(memberNames[1]);
        result.append(")");
        return result.toString();
	}
	
	public static PositionUnionExpression fromString ( String sparm ) throws ElementParseException
	{

		try
		{
			FunctionExpressionParser fep = new FunctionExpressionParser(sparm);
			
			List expressionArgs = fep.getArguments();
			
			if ( expressionArgs.size() != 2 )
			{
				throw new ElementParseException("Union expression taking " + expressionArgs.size() + " arguments when 2 are expected");
			}
			
			Element argument1 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(0)));
			Element argument2 = ElementParser.getCurrentInstance().parse(((String)expressionArgs.get(1)));
			
			return new PositionUnionExpression ( argument1 , argument2 );
			
		}
		catch ( FunctionExpressionParseException fepe )
		{
			throw new ElementParseException(fepe);
		}
		
	}
	
}
