/*
 * Created on 10-ago-2004 16:05:08
 *
 */
package generator;

import element.CompositeElement;
import java.util.*;
import element.SimpleElement;
import element.ExpressionElement;
import element.SequenceElement;


/**
 * <p>Este visitante genera el cdigo necesario para asignar a los atributos de un
 * paso deductivo los valores asociados a un item que hemos buscado en el ItemHandler,
 * y que no estuviesen definidos ya.</p>
 * <p>Como se puede ver, tiene caractersticas parecidas a las del ConstructorAssignVisitor
 * y otras que recuerdan ms a las del ApplyTriggerMatchVisitor.</p>
 * <p>Created 10-ago-2004 16:05:08</p>
 * @author carlos
 */
public class ApplyNonTriggerAssignVisitor extends CodeGeneratingVisitor
{

	/**
	 * En este buffer se almacena el cdigo que devuelve el paso deductivo a su
	 * estado inicial si descubrimos que no hay matching cuando ya se haban llevado
	 * a cabo algunas asignaciones.
	 * Este recovery code es local, slo deshace lo correspondiente al non trigger
	 * assign visitor.
	 */
	private StringBuffer recoveryCode = new StringBuffer("");
    
    /**
     * Lista para poder deshacer los cambios en el estado de los atributos que
     * pasemos a ground.
     */
    private List groundedAttrs = new ArrayList();

    /**
     * Genera nmeros para aadir al nombre de las variables enteras que se utilizan
     * como ndices para los bucles de emparejamiento de los elementos Sequence. De este
     * modo, si emparejamos varios Sequence, podemos dar nombres distintos a las
     * variables.
     */
    private static int tempVariableIndex = 0;
    
    /**
     * A esta cadena se le dar un valor cuando este visitante visite un elemento
     * de tipo Sequence. Este valor ser el nombre de la variable que contiene
     * el nmero de items que han emparejado con el Sequence. Tal nombre es necesario
     * almacenarlo para que leerlo cuando estemos visitando el CompositeElement que
     * contiene el Sequence, pues lo utilizaremos para aadirlo al ndice del array.
     */
    private String indexIncrement = null;
    
    /**
     * Devuelve un nombre para una variable temporal. Se garantiza que en cada
     * llamada se devuelve un nombre distinto.
     * @return El nombre para una variable temporal.
     */
    private static String nextTempVariableName()
    {
    	return ( "mmatch_temp_" + (tempVariableIndex++) );
    }
    
    /**
     * Establece el cdigo de recuperacin, que normalmente vendr de un
     * ApplyNonTriggerAssignVisitor llamado anteriormente.
     * @param sb El buffer que contiene el cdigo de recuperacin.
     */
    public void setRecoveryCodeBuffer ( StringBuffer sb )
    {
        this.recoveryCode = sb;
    }
    
	/**
	 * Obtiene el buffer donde se almacena el cdigo de recuperacin.
	 * Este cdigo tiene que ser utilizado por el generador de cdigo ms adelante
	 * @return Buffer con cdigo de recuperacin del estado del paso deductivo para 
	 * contrarrestar lo que hace el cdigo generado por este visitante, en caso
	 * de que no haya matching.
	 */
	public StringBuffer getRecoveryCodeBuffer()
	{
	    return recoveryCode;
	}
    
    /**
     * @param objectName El nombre de la variable que contiene el item.
     * @param toAppendTo Buffer en el que se insertar el cdigo generado.
     * @param cg Generador de cdigo que contiene datos de atributos.
     */
    public ApplyNonTriggerAssignVisitor ( String objectName, StringBuffer toAppendTo, DeductiveStepCodeGenerator cg  )
    {	
        super ( objectName, toAppendTo, cg );
        //assign this item to a variable to avoid repeated gets (optimization)
        String varName = cg.nextItemVariableName();
        toAppendTo.append( "Object [] " + varName + " = " + objectName + ";\n");
        this.objectName = varName;
    }

    /**
     * @param se Elemento simple al que queremos asignar un valor obtenido de un item concreto.
     */
    public void visitSimpleElement ( SimpleElement se )
    {
		Attribute attr = generator.attributeFor ( se );
		//assert: attr should be != null
		if ( attr != null && !generator.isGround(attr) )
		{
		
		    //Asignamos al atributo su valor en el item, hacindolo ground (definido).
		    
		    //Al igual que en el ApplyTriggerMatchVisitor, hemos de chequear la clase
		    //de lo que se nos devuelve, pues la clase podra no coincidir al no ir
		    //especificada en el esqueleto. Asimismo, hemos de tener en cuenta que
		    //la asignacin que generamos podr ser contrarrestada luego si el matching
		    //falla (o simplemente al final de la funcin si va bien) con lo cual tendremos
		    //que generar ms cdigo de recuperacin.
		    
	        toAppendTo.append(attr.getClassName() + " " + "old_" + attr.getName() + " = " + attr.getName() + ";\n" );
	        
	        //agregar asignacin inversa al cdigo de recuperacin
	        recoveryCode.insert(0, attr.getName() + " = " + "old_" + attr.getName() + ";\n" );
		    
	        //cdigo que intenta llevar a cabo la asignacin y, en caso de que
	        //las clases no coincidan con lo esperado,
	        //llama al cdigo de recuperacin.
	        
	        toAppendTo.append ( "if ( " + objectName + " instanceof " + attr.getClassName() + " )\n" );
	        toAppendTo.append ( "{\n" );
	        toAppendTo.append ( attr.getName() + " = " + "( " + attr.getClassName() + " )" + " " + objectName + ";\n" );
	        toAppendTo.append ( "} //class check passed\n" );
	        toAppendTo.append ( "else\n" );
	        toAppendTo.append ( "{\n" );
	        toAppendTo.append ( recoveryCode.toString() );
	        //[removed 05.10.07] toAppendTo.append ( "return result;\n" );
	        //toAppendTo.append( "continue;\n" ); //[added 05.10.07]
	        //toAppendTo.append ( "return result;\n" );
	        toAppendTo.append( "continue;\n" );
	        
	        toAppendTo.append ( "} //class check failed\n" );
	        
	        
	        generator.setGround ( attr , true );
	        groundedAttrs.add(attr);
	        		
		}
		/*
		 * Qu pasa si no se cumple el if y el atributo es ground? No nos preocupamos
		 * de nada, porque si era ground entonces ya lo hemos incluido en el esqueleto
		 * como parte del "query" al ndice, y la manera en que funciona ItemHandler
		 * nos garantiza que el valor dado en objectName va a coincidir con el esperado
		 * en el atributo.
		 * Ntese que esto va a ser diferente para los ExpressionElements, que s que
		 * tendremos que chequear, pues puede que se hayan convertido en "ground" durante
		 * este mismo proceso de asignacin y que el valor asociado no coincide con el
		 * esperado. Por ejemplo, podemos estar buscando items de la forma [B,j,j+1] pero
		 * donde slo es conocido B, y j es cualquier posicin (i.e. j es un atributo
		 * no ground y j+1 una expresin no ground, en el esqueleto iran dos nulos).
		 * Si encontramos el item [B,4,7], que es vlido para ese esqueleto, al
		 * hacer el match del 4 la expresin se hace ground, y tenemos que darnos cuenta
		 * de que no es la que buscamos al analizar el 7.
		 * [04-10-27] Aparte del anteriormente expuesto, hay otro motivo para chequear
		 * siempre los ExpressionElements, incluso si las expresiones que estamos
		 * manejando ya eran "ground" antes de la asignacin, y es que un cdigo de
		 * expresin (getExpressionCode()) puede no estar creando un elemento vlido.
		 * Esto sucede, por ejemplo, si tenemos una cadena de longitud 5 (en cuyo caso
		 * las StringPosition vlidas sern 0,1,2,3,4,5) y nos ponemos a buscar un
		 * item de la forma [B,j,j+1], siendo j=5, con el Earley Predictor.
		 *  
		 */
		
    }
    
	/**
	 * Esta funcin hace que los atributos que habamos fijado a ground en el generador
	 * de cdigo vuelvan a no-ground, para poder generar otros mtodos.
	 * Tiene que ser llamada desde fuera.
	 */
	public void undoChanges()
	{
	    for ( int i = 0 ; i < groundedAttrs.size() ; i++ )
	    {
	        generator.setGround ( (Attribute)groundedAttrs.get(i) , false );
	    }
	}

    /**Flag que slo se activa en la primera llamada al visitante, que ser la ms externa, despus estar a false
     * para que el resto de llamadas recursivas.*/
    private boolean firstTimeFlag = true;
	
	/**
	 * En este caso no se hace nada para los elementos compuestos salvo pasar a los hijos,
	 * pues las asignaciones en principio sern siempre de objetos elementales.
	 */
    public void visitCompositeElement ( CompositeElement ce )
    {
    	
    	//check desired length.
		//la longitud mnima deseada es el nmero de elementos del composite que
		//no son Sequence
		int desiredLength = ce.numChildren();
		for ( int i = 0 ; i < ce.numChildren() ; i++ )
		{
			if ( ce.getChild(i) instanceof SequenceElement )
				desiredLength--;
		}
		if ( desiredLength == ce.numChildren() )
		{
			toAppendTo.append ( "if ( ((Object[])" + objectName + ").length != " + ce.numChildren() + " ) \n" );
			toAppendTo.append ( "{\n" );
	        toAppendTo.append ( recoveryCode.toString() );
	        toAppendTo.append ( "continue;\n" );
	        toAppendTo.append ( "}\n" ); 
		}		
		else
			{
			toAppendTo.append ( "if ( ((Object[])" + objectName + ").length < " + desiredLength + " ) \n" );
			toAppendTo.append ( "{\n" );
	        toAppendTo.append ( recoveryCode.toString() );
	        toAppendTo.append ( "continue;\n" );
	        toAppendTo.append ( "}\n" );
			}
    	
    	
		String oldObjectName = this.objectName;
		indexIncrement = null;
		String increments = "";
		
		boolean externalFlag = firstTimeFlag;
		firstTimeFlag = false;
		
		int nonZeroOrMoreChildrenProcessed = 0; //processed since last setting of variable increments
		for ( int i = 0 ; i < ce.numChildren() ; i++ )
		{	
			if ( indexIncrement != null )
			{
				/**
				 * Si entramos en este if, es que el ltimo hijo procesado era un
				 * Sequence (multimatcher). En este caso, no debemos incrementar
				 * el ndice en 1, pues no sabemos en tiempo de generacin de cdigo
				 * si emparejaremos con un elemento, con cero o con ms de uno.
				 * El nombre de la variable que contendr el nmero de elementos con
				 * que emparejamos lo guarda el mtodo visitSequenceElement de 
				 * este mismo visitante en la variable indexIncrement. As pues,
				 * tenemos que sumar ese nombre a nuestro ndice en el cdigo generado.
				 */
				//increments+=("+"+indexIncrement);
				increments="+"+indexIncrement; //meaning of indexIncrement is different
				//here from that in ApplyTriggerMatchVisitor
				nonZeroOrMoreChildrenProcessed = 0;
				indexIncrement = null;
			}
			if ( externalFlag )
				this.objectName = oldObjectName + "["+nonZeroOrMoreChildrenProcessed+increments+"]";
			else
				this.objectName = "((Object[])" + oldObjectName + ")" + "[" + nonZeroOrMoreChildrenProcessed + increments + "]";
			ce.getChild(i).accept(this);
			if ( !(ce.getChild(i) instanceof SequenceElement ))
			{
				nonZeroOrMoreChildrenProcessed++;
			}
		}
		
		//BEGIN code added 2005.12.19
		//if ( indexIncrement != null )
		//{
		if ( desiredLength != ce.numChildren() )
		{
			toAppendTo.append("//begin nontrigger composite element final length check\n");
			/*
			 * Necesitamos un chequeo final de longitud, porque podemos haber hecho
			 * pattern-matching con todo pero que todava nos queden cosas en el item.
			 */
			
			//obtener el indice a explorar "siguiente al ultimo"
			//notese que si hubiese un elemento en el composite siguiendo al ultimo que
			//se explora, su indice seria la longitud deseada.
			
			if ( indexIncrement != null )
			{
				increments="+"+indexIncrement;
				nonZeroOrMoreChildrenProcessed = 0;
				indexIncrement = null;
			}
			if ( externalFlag )
				this.objectName = oldObjectName + "["+nonZeroOrMoreChildrenProcessed+increments+"]";
			else
				this.objectName = "((Object[])" + oldObjectName + ")" + "[" + nonZeroOrMoreChildrenProcessed + increments + "]";
		
			int lastOpenBracket = objectName.lastIndexOf("[");
	        int closedBracket = objectName.indexOf("]",lastOpenBracket);
	        //String beforeIndex = objectName.substring(0,lastOpenBracket+1);
	        String index = objectName.substring(lastOpenBracket+1,closedBracket);
	        //String afterIndex = objectName.substring(closedBracket);
			
			
			String tmpname = (externalFlag)? (oldObjectName) : ("((Object[])"+oldObjectName+")"); 
			toAppendTo.append ( "if ( " + tmpname + ".length != " +index+ " )\n" );
	        toAppendTo.append ( "{\n" );
	        toAppendTo.append ( recoveryCode.toString() );
	        toAppendTo.append ( "continue;\n" );
	        toAppendTo.append ( "}\n" ); 
			indexIncrement = null; //Si el ltimo elemento visitado ha sido un Sequence,
			//no queremos que esto afecte al llamador externo.
			toAppendTo.append("//end nontrigger composite element final length check\n");
		//}
		}
		//END code added 2005.12.19
		
		
		indexIncrement = null; //Si el ltimo elemento visitado ha sido un Sequence,
		//no queremos que esto afecte al llamador externo.
		this.objectName = oldObjectName; //dejamos el visitante como estaba.
    }
    
    
    /**
     * En el caso de un elemento de tipo Sequence, tenemos que generar cdigo
     * para ir aadiendo al atributo asociado a ese elemento, que ser una lista,
     * todos los elementos simples de la clase correspondiente a ese Sequence
     * que nos encontremos a partir de objectName (el propio objectName y los
     * siguientes, obtenidos modificando el ndice de array de objectName).
     * @param zome
     */
    public void visitSequenceElement ( SequenceElement zome )
    {
    	Attribute attr = generator.attributeFor ( zome );
    	//{attr not null y attr.getElementClass() debera ser alguna subclase de List}
    	if ( !generator.isGround(attr) )
    	{
    		
	        //agregar asignacin inversa al cdigo de recuperacin.
    		//En este caso no necesitamos variable old, sabemos que si el OneOrMore
    		//era nonground la lista iba a estar vaca.
	        
		    recoveryCode.insert(0, attr.getName() + ".clear();\n");
    		
    		
	        //Tomar el ndice de objectName.
	        //Aqu partimos la cadena objectName para separar en partes
	        //el ndice del array, la parte anterior, la parte posterior y el nombre
	        //del array. Por ejemplo, si tenemos
	        //miItem[k][6][i+1] => hacemos:
	        //beforeIndex = miItem[k][6][
	        //index = i+1
	        //afterIndex = ]
	        //arrayName = miItem[k][6] (el arrayName se usa para ver la longitud).
	        int lastOpenBracket = objectName.lastIndexOf("[");
	        int closedBracket = objectName.indexOf("]",lastOpenBracket);
	        String beforeIndex = objectName.substring(0,lastOpenBracket+1);
	        String index = objectName.substring(lastOpenBracket+1,closedBracket);
	        String afterIndex = objectName.substring(closedBracket);
	        String arrayName = beforeIndex.substring(0,beforeIndex.length()-1) + afterIndex.substring(1);
	        
	        //obtenemos el nombre para el ndice de bucle de este multimatcher.
	        String loopIndexName = nextTempVariableName();
	        
	        //Generamos el cdigo de un bucle que ir recorriendo elementos y aadindolos
	        //a la lista del multimatcher mientras sean de la clase asociada a ste.
	        //Ntese que en este caso no necesitamos estructuras try-catch
	        //ni cdigo de recuperacin, porque el matching de un Sequence no puede
	        //fallar: en el caso de que nos encontremos elementos cuya clase no
	        //coincide con la esperada por el Sequence, simplemente ste habr
	        //emparejado con cero elementos; pero empareja correctamente.
	        
	        
	        //REMOVED this line @ 05.12.22:
	        //toAppendTo.append ( attr.getName() + ".clear();\n" );
	        //We don't need it. As an applyTo precondition, the array lists are
	        //clear. They are cleared at the recovery code which is always execced
	        //before returning an item list (even an empty one)
	        
	        toAppendTo.append ( "int " + loopIndexName + " = " + index + ";\n");
	        toAppendTo.append ( "while ( " + loopIndexName + " < " + arrayName + ".length && " + beforeIndex + loopIndexName + afterIndex + " instanceof " + zome.getAssociatedClass().getName() + " )\n" );
	        toAppendTo.append ( "{\n" );
	        toAppendTo.append ( attr.getName() + ".add(" + beforeIndex + loopIndexName + afterIndex + ");\n" );
	        toAppendTo.append ( loopIndexName + "++;\n");
	        toAppendTo.append ( "}\n");
	        
	        //Seteamos indexIncrement, utilizado por el mtodo visitCompositeElement
	        //que llama a ste.
	        indexIncrement = loopIndexName;
	        
	        //Indicamos en el generador de cdigo que la lista ya es ground.
	        generator.setGround ( attr , true );
	        groundedAttrs.add(attr);
    		
    	}
    	//Si el Sequence es ground, suponemos que la informacin de los
    	//elementos concretos que hay en l ya iba en el esqueleto, por lo tanto
    	//nos tienen que haber devuelto esos elementos y no tenemos por qu
    	//chequear nada.
    	
    	//BEGIN CODE ADDED 05.12.20
    	else
    	{
    		//ground sequence? this appears in situations like
    		// [ A -> alpha . , ... ]
    		// [ A -> alpha . , ... ]
    		//We don't have to fill the list because we have it filled already.
    		//But we have to match in order to test the alphas for equality.
    		
//    		//Tomar el ndice de objectName.
	        //Aqu partimos la cadena objectName para separar en partes
	        //el ndice del array, la parte anterior, la parte posterior y el nombre
	        //del array. Por ejemplo, si tenemos
	        //miItem[k][6][i+1] => hacemos:
	        //beforeIndex = miItem[k][6][
	        //index = i+1
	        //afterIndex = ]
	        //arrayName = miItem[k][6] (el arrayName se usa para ver la longitud).
	        int lastOpenBracket = objectName.lastIndexOf("[");
	        int closedBracket = objectName.indexOf("]",lastOpenBracket);
	        String beforeIndex = objectName.substring(0,lastOpenBracket+1);
	        String index = objectName.substring(lastOpenBracket+1,closedBracket);
	        String afterIndex = objectName.substring(closedBracket);
	        String arrayName = beforeIndex.substring(0,beforeIndex.length()-1) + afterIndex.substring(1);
	        
	        //obtenemos el nombre para el ndice de bucle de este multimatcher.
	        String loopIndexName = nextTempVariableName();
    		String boolVarName = nextTempVariableName();
    		
    		toAppendTo.append ( "boolean " + boolVarName + " = true;\n");
    		toAppendTo.append ( "int " + loopIndexName + " = " + index + ";\n");
    		toAppendTo.append ( "int ini" + loopIndexName + " = " + index + ";\n");
    		toAppendTo.append ( "while ( " + loopIndexName + " < " + arrayName + ".length && " + beforeIndex + loopIndexName + afterIndex + " instanceof " + zome.getAssociatedClass().getName() + " )\n" );
	        toAppendTo.append ( "{\n" );
	        toAppendTo.append ( "if ( !" + beforeIndex + loopIndexName + afterIndex + ".equals(" + attr.getName()+ ".get(" + loopIndexName + "-" + "ini" + loopIndexName + ") ) )\n" );
            toAppendTo.append("{\n");
            toAppendTo.append ( boolVarName + " = false;\n");
            toAppendTo.append("break; //to continue outside loop\n");
            toAppendTo.append("}\n");
	        toAppendTo.append ( loopIndexName + "++;\n");
	        toAppendTo.append ( "}\n");
	        toAppendTo.append( "if ( !" + boolVarName + " )");
	        toAppendTo.append("{\n");
            toAppendTo.append ( recoveryCode.toString() );
            toAppendTo.append("continue; //to next item in list\n");
            toAppendTo.append("}\n");
	        
	        //Seteamos indexIncrement, utilizado por el mtodo visitCompositeElement
	        //que llama a ste.
	        indexIncrement = loopIndexName;
	        
    	}
    	//END CODE ADDED 05.12.20
    	
    }


       
    /**
     * En el caso de una expresin, slo tratamos expresiones que sean ground, y lo
     * que hacemos es verificar si realmente el item obtenido se ajusta al valor
     * esperado para dicha expresin, de lo contrario no nos vale y nos olvidamos
     * de ese item.
     * @param ee Elemento que representa una expresin en un paso deductivo.
     */
    public void visitExpressionElement ( ExpressionElement ee )
    {
        
        
        if ( generator.isGround(ee) )
        {
            //chequear que el valor coincide con lo esperado.
            //La necesidad de este chequeo se explica en un comentario de
            //visitSimpleElement.
            toAppendTo.append("if ( ");
            toAppendTo.append("!"+objectName+".equals( ");
            toAppendTo.append(generator.generateExpressionCode(ee));
            toAppendTo.append(" ) )");
            toAppendTo.append("{\n");
            toAppendTo.append ( recoveryCode.toString() );
            toAppendTo.append("continue; //to next item in list\n");
            toAppendTo.append("}\n");
        }
        else
        {
            /*
             * Hemos pedido al ItemHandler un item basndonos en un formato en el que
             * haba una expresin, nos han devuelto el item pero no podemos resolver
             * la expresin. Esto se debe a un fallo estructural en el paso deductivo, a que
             * los items del antecedente estn puestos en un mal orden (p.ej [i+1] antes
             * que [i]), etc.) y tal vez debera tratarse mediante una excepcin.
             */
           System.err.println("[ERROR] Unresolved expression at item obtained from skeleton, " + generator.generateExpressionCode(ee) );
        }
        
        
		

		
    }
    
    
}
