/*
 * Created at regulus on 06-nov-2004 17:32:12
 * as file SearchDescriptorMatchCodeGenerator.java on package generator
 */
package generator;

import element.SequenceElement;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;


public class SearchDescriptorMatchCodeGenerator
{
 
	private String objectName;
    
	//si el generador est en estado "alpha-a", es decir, estamos emparejando una
	//cadena que no debera incluir el ltimo smbolo.
	private boolean alpha_a_flag = false;
	
	
    /**
     * <p>Esta tabla hash permite pasar direcciones en el descriptor de bsqueda a direcciones en
     * el tem de los objetos a utilizar en el ndice.</p>
     * <p>Las direcciones en el tem se dan en forma de cdigo, de este modo: ((Object[])item)[2].</p>
     * <p>Por qu es necesario hacer esto? Ser necesario cuando el descriptor de bsqueda
     * contenga nodos asociados a la clase SequenceElement. Por ejemplo:</p>
     *   B -> gamma , j , k (en Earley Completer)
     *   SearchDescriptorNode: Object[]
     *     SearchDescriptorNode: Symbol
     *     SearchDescriptorNode: SequenceNode
     *       SearchDescriptorNode: Symbol
     *     SearchDescriptorNode: IntElt
     *     SearchDescriptorNode: IntElt
     * <p>Si queremos indexar por el entero j, no podemos simplemente usar item[2], el
     * ndice a usar depender de cuntos smbolos fuesen emparejados por la secuencia gamma
     * (una variable en el cdigo). As, el addressConverter nos deber devolver
     * algo como [mmatchind+2] o similar, donde mmatchind es una variable del cdigo generado.</p>
     */
    private HashMap addressConverter;
    
    /**
     * Obtiene la tabla de conversin de direcciones.
     */
    public HashMap getAddressConverter()
    {
        return addressConverter;
    }
    
    /**
     * Crea una instancia de SearchDescriptorMatchCodeGenerator para generar cdigo
     * que chequee el objeto de nombre dado contra diferentes descriptores de bsqueda
     * (que se darn ms tarde, en el mtodo generateCode).
     * @param objectName Nombre que tendr en el cdigo la variable o parmetro
     * que almacena el objeto que va a comprobar este generador.
     */
    public SearchDescriptorMatchCodeGenerator ( String objectName  )
    {
        this.objectName = objectName;
    }
    
    private static int lastIndexVarNumber = 0;
    
    /**
     * Devuelve un nombre de variable no usado antes para un ndice.
     */
    private static String getNewIndexVarName()
    {
    	return ("ind" + (lastIndexVarNumber++));
    }
    
    /**
     * Genera el cdigo que comprueba si un objeto (cuyo nombre vendr dado
     * por el atributo objectName) corresponde al rbol de esqueleto dado.
     * Supone que existe una variable booleana "skelmatch" declarada en el cdigo.
     */
    public String generateCode ( SearchDescriptorNode stn , boolean forAddItemFunction , boolean nullsAllowedAtInput )
    {
        addressConverter = new HashMap();
        List currentAddress = new ArrayList();
        String currentObjectName = objectName;
        
        StringBuffer result = new StringBuffer("");
        StringBuffer decls = new StringBuffer();
        StringBuffer code = new StringBuffer ( doGenerateCode ( stn , currentAddress , currentObjectName , forAddItemFunction , nullsAllowedAtInput , decls ) );
        result.append(decls);
        result.append(code);
        return result.toString();
    }

    private String increments;
    
    //ntese que debe existir variable skelmatch, que esto deja a true o false segn si hay match o no.
    /**
     * Mtodo auxiliar que se llama recursivamente para hacer el trabajo de generateCode.
     * (generateCode genera cdigo para comprobar si un item se corresponde con un descriptor de bsqueda dado, 
     * y crea el conversor de direcciones).
     * @param stn Nodo del rbol de esqueleto que estamos recorriendo actualmente.
     * @param currentAddress Direccin asociada a dicho nodo con respecto a la raz
     * del descriptor de bsqueda. Estas direcciones se irn introduciendo en el conversor
     * de direcciones (atributo addressConverter) asociado a este generador. Al final
     * de la llamada a generateCode(), tendremos en dicho atributo un mapping entre
     * direcciones del descriptor de bsqueda y cdigo para acceder a los objetos
     * del programa generado que corresponden a dichas direcciones. (ntese que
     * este atributo addressConverter se resetea al principio de cada llamada
     * a generateCode()).
     * @param currentObjectName Nombre con el que hacemos referencia en el cdigo
     * generado al objeto correspondiente al nodo del descriptor de bsqueda que
     * estamos recorriendo.
     * @param forExistsOrAddItemFunction si la funcin cuyo cdigo generamos es addItem o exists.
     * (ntese que addStep de StepHandler no la usa)
     * @param outInitialDeclarations Parmetro de salida, en l quedan las declaraciones
     * de las variables ndice que aparecen en este cdigo, y tendrn que ser aadidas
     * al principio del mtodo donde est.
     * @param nullsAllowedAtInput Si el objeto manejado por el cdigo (objectName) puede
     * contener nulos. Esto sucede en StepHandler, donde en lugar de tems estamos indexando
     * peticiones de bsqueda.
     */
    public String doGenerateCode ( SearchDescriptorNode stn , List currentAddress , String currentObjectName , boolean forExistsOrAddItemFunction , boolean nullsAllowedAtInput , StringBuffer outInitialDeclarations )
    {
        StringBuffer result = new StringBuffer("");
        
        if ( outInitialDeclarations == null )
        	outInitialDeclarations = new StringBuffer("");
        
        //clonamos la direccin actual para ponerla en el conversor.
        //(la actual se va a modificar y el efecto no sera el deseado)
        List clonedAddress = new ArrayList();
        for ( int i = 0 ; i < currentAddress.size() ; i++ )
            clonedAddress.add(currentAddress.get(i));
        
        /* 
         para el conversor de direcciones necesitamos que el cdigo resultante
         haga referencia a un objeto castendolo, dado que llamaremos sobre l a mtodos
         como getArrayIndex().
        */
        String castedCurrentObjectName;
        if ( stn.getAssociatedClass() != null )
            castedCurrentObjectName = "((" + ((stn.getAssociatedClass().isArray())?("Object[]"):(stn.getAssociatedClass().getName())) + ")" + currentObjectName + ")";
        else
            castedCurrentObjectName = currentObjectName;
        
        addressConverter.put(clonedAddress,castedCurrentObjectName);
                              
        //null skeleton element, may be any simple element
        if ( stn.getAssociatedClass() == null )
        {
            return "";
        }
        
        /*
         * Este cdigo es vlido para elementos simples y composites; pero
         * NO para elementos tipo "secuencia".
         * Chequeo de clase, generamos el cdigo que chequea
         * currentObjectName es la clase asociada al nodo actual.
         * (ntese que se tienen en cuenta los arrays mediante un condicional)
         */
        if ( !stn.getAssociatedClass().equals(SequenceElement.class) )
        {
        	
        	/*
        	 * Flag "nullsAllowedAtInput": este flag estar a true cuando estemos
        	 * generando el cdigo para aadir pasos al ndice en la StepHandler.
        	 * En este caso, lo que estamos aadiendo a los ndices no son items
        	 * ground (como en addItem) sino peticiones de bsqueda. Estas peticiones de bsqueda pueden ser
        	 * ms generales que el tipo de peticin esperada (i.e. contener un nulo
        	 * donde nosotros esperbamos algo de una clase concreta) y nos interesa
        	 * aadirlos de todos modos, de ah que aceptemos null como que matchea
        	 * con cualquier cosa.
        	 * Ntese que esto no interesa en el mtodo addItem de ItemHandler, pues
        	 * los items que aadimos sern siempre ground y sin nulos. Tampoco interesa
        	 * en getBySpecification, porque si la entrada es una peticin de bsqueda general no queremos
        	 * que se nos devuelvan los items que cumplan uno ms particular, podramos
        	 * dejar items fuera.
        	 */
        	if ( nullsAllowedAtInput )
        		result.append("if ( skelmatch && ("+ currentObjectName + " != null && !(" + currentObjectName + " instanceof " + (stn.getAssociatedClass().isArray()?"Object[]":stn.getAssociatedClass().getName()) + ") ))\n");
        	else
        		result.append("if ( skelmatch && !(" + currentObjectName + " instanceof " + (stn.getAssociatedClass().isArray()?"Object[]":stn.getAssociatedClass().getName()) + ") )\n");
	        result.append("{\n");
	        result.append("skelmatch=false;\n");
	        result.append("} //if [class doesn't match]\n");
        } //not SequenceElement [class check]
        
        
        if ( stn.getAssociatedClass().equals(SequenceElement.class) )
        {
        	if  (!forExistsOrAddItemFunction)
        	{
        		//chequeo de clase, como el de elementos que no son Sequence.
        		//(estamos esperando un Sequence)
                result.append("if ( skelmatch && !(" + currentObjectName + " instanceof " + (stn.getAssociatedClass().isArray()?"Object[]":stn.getAssociatedClass().getName()) + ") )\n");
                result.append("{\n");
                result.append("skelmatch=false;\n");
                result.append("} //if [class doesn't match]\n");
        	}
        	else
        	{
    	        //Tomar el ndice de currentObjectName.
    	        //Aqu partimos la cadena currentObjectName para separar en partes
    	        //el ndice del array, la parte anterior, la parte posterior y el nombre
    	        //del array. Por ejemplo, si tenemos
    	        //miItem[k][6][i+1] => hacemos:
    	        //beforeIndex = miItem[k][6][
    	        //index = i+1
    	        //afterIndex = ]
    	        //arrayName = miItem[k][6] (el arrayName se usa para ver la longitud).
    	        int lastOpenBracket = currentObjectName.lastIndexOf("[");
    	        int closedBracket = currentObjectName.indexOf("]",lastOpenBracket);
    	        String beforeIndex = currentObjectName.substring(0,lastOpenBracket+1);
    	        String index = currentObjectName.substring(lastOpenBracket+1,closedBracket);
    	        String afterIndex = currentObjectName.substring(closedBracket);
    	        String arrayName = beforeIndex.substring(0,beforeIndex.length()-1) + afterIndex.substring(1);
    	        
    	        //variable ndice
    	        String indexVarName = getNewIndexVarName();
    	        
    	        outInitialDeclarations.append("int " + indexVarName + " = 0;\n" );
    	        result.append(indexVarName + " = 0;\n");    	        
    	        
    	        //Cdigo de soporte del "alpha-a matching" (que una Sequence no se trague todos los smbolos que
    	        //pueda si a continuacin viene un Symbol)
    	        if ( alpha_a_flag )
    	        {
    	        	/**
    	        	 * Si estamos en estado "alpha-a", quiere decir que estamos tratando
    	        	 * con una cadena (multimatcher/sequence) seguida de un elemento del mismo
    	        	 * tipo que la cadena, es decir, no debemos "tragar" el ltimo
    	        	 * elemento. Esto lo resolvemos con un bucle que comprueba la clase
    	        	 * no slo del elemento actual, sino del siguiente.
    	        	 */
    	        	result.append("if (skelmatch)\n");
        	        result.append("while ( " + "(" + index + "+" + indexVarName + "+ 1 <" + arrayName + ".length" + ") && (" + beforeIndex + index + "+" + indexVarName + afterIndex + " instanceof " + stn.getChild(0).getAssociatedClass().getName() + ") && (" + beforeIndex + index + "+1+" + indexVarName + afterIndex + " instanceof " + stn.getChild(0).getAssociatedClass().getName() + ")" + ") " + indexVarName + "++;\n" );
        	        increments = increments + "+"+indexVarName;
    	        }
    	        else
    	        {
    	        	/**
    	        	 * Si no estamos en estado "alpha-a", el matching de la cadena
    	        	 * consiste simplemente en coger tantos elementos de la clase
    	        	 * asociada como podamos encontrar.
    	        	 */
    	        	result.append("if (skelmatch)\n");
        	        result.append("while ( " + "(" + index + "+" + indexVarName + "<" + arrayName + ".length" + ") && (" + beforeIndex + index + "+" + indexVarName + afterIndex + " instanceof " + stn.getChild(0).getAssociatedClass().getName() + ")" + ") " + indexVarName + "++;\n" );
        	        increments = increments + ("+"+indexVarName);
    	        }
    	        
    	        
        	}
        } //SequenceElement [sequence matching]
        
        /*
         * Este cdigo es slo para los nodos composite, asociados al tipo Object[].
         * Generamos cdigo para hacer un chequeo de longitud y adems comprobar
         * recursivamente los hijos.
         */
        if ( stn.getAssociatedClass().isArray() )
        {
        	
        	boolean variableLength = false;
        	int thelen = 0;
        	int SequenceElementsLeft = 0;
        	if ( forExistsOrAddItemFunction )
        	{
        		for ( int i = 0 ; i < stn.numChildren() ; i++ )
        		{
        			if ( stn.getChild(i).getAssociatedClass() != null && stn.getChild(i).getAssociatedClass().equals(SequenceElement.class))
        			{
        				variableLength = true;
        				SequenceElementsLeft++;
        			}
        			else
        			{
        				thelen++;
        			}
        		}
        	}
        	
        	if ( !variableLength )
        	{
        		if ( nullsAllowedAtInput )
        			result.append("else if ( skelmatch && ( " + currentObjectName + "== null || " + castedCurrentObjectName + ".length != " + stn.numChildren() + " ) )\n");
        		else
        			result.append("else if ( skelmatch && " + castedCurrentObjectName + ".length != " + stn.numChildren() + ")\n");
        	}
        	else
        	{
        		if ( nullsAllowedAtInput )
        			result.append("else if ( skelmatch && ( " + currentObjectName + "== null || " + castedCurrentObjectName + ".length < " + thelen + ") )\n");
        		else
        			result.append("else if ( skelmatch && " + castedCurrentObjectName + ".length < " + thelen + ")\n");
        	}
            result.append("{\n");
            result.append("skelmatch=false;\n");
            result.append("} //if [length doesn't match]\n");
            result.append("else\n");
            result.append("{\n");
            //check children
            increments = "";
            int i = 0 , j = 0;
            for ( i = 0 , j = 0 ; i < stn.numChildren() ; i++ )
            {
                currentAddress.add( new Integer(i) );
                String newObjectName = "((Object[])" + currentObjectName + ")[" + j + increments + "]";
                
                String tempIncrements = increments;
                increments = "";
                
                //caso alpha-a

                //if ( stn.getChild(i).getAssociatedClass() == (SequenceElement.class) && i < stn.numChildren() - 1 && stn.getChild(i+1).getAssociatedClass().equals((stn.getChild(i).getChild(0)).getAssociatedClass()))
                //	alpha_a_flag = true;                
                // 05.10.09 change: subject of equals could be null, causing an exception to be thrown.
                // 05.10.10 change: we add the or.
                if ( stn.getChild(i).getAssociatedClass() == (SequenceElement.class) && i < stn.numChildren() - 1 && ( stn.getChild(i+1).getAssociatedClass() == (stn.getChild(i).getChild(0)).getAssociatedClass() || stn.getChild(i+1).getAssociatedClass() == null )  )
                	alpha_a_flag = true; 
                
                result.append(doGenerateCode(stn.getChild(i),currentAddress,newObjectName,forExistsOrAddItemFunction,nullsAllowedAtInput,outInitialDeclarations));
                
                alpha_a_flag = false;
                
                increments = tempIncrements+increments;
                
                currentAddress.remove( currentAddress.size()-1 );
                if ( stn.getChild(i) == null || stn.getChild(i).getAssociatedClass() != SequenceElement.class || !forExistsOrAddItemFunction )
                {
                	j++;
                }
                else
                {
                	SequenceElementsLeft--;
                	//chequeo intermedio de longitud despus de secuencia
                	if ( SequenceElementsLeft > 0 )
                	{
                		if ( nullsAllowedAtInput )
                			result.append("if ( skelmatch && ( " + currentObjectName + "== null || " + castedCurrentObjectName + ".length < " + thelen + increments + ") )\n");
                		else
                			result.append("if ( skelmatch && " + castedCurrentObjectName + ".length < " + thelen + increments + ")\n");
                	}
                	else //ltimo sequence, el match de la longitud tiene que ser exacto (i.e. que no haya cosas de ms)
                	{
                		if ( nullsAllowedAtInput )
                			result.append("if ( skelmatch && ( " + currentObjectName + "== null || " + castedCurrentObjectName + ".length != " + thelen + increments + ") )\n");
                		else
                			result.append("if ( skelmatch && " + castedCurrentObjectName + ".length != " + thelen + increments + ")\n");
                	}
                	result.append("{\n");
                	result.append("skelmatch=false;\n");
                	result.append("} //if [length doesn't match, intermediate check]\n");
                	result.append("else\n");
					result.append("{\n");
                }
            }
            for ( ; j < i ; j++ )
            {
            	result.append("} //else [length does match, intermediate check]\n");
            }
            result.append("} //else [if length ok]\n");
            
            increments = ""; //si somos un composite, no dejamos basura en increments.
            
        } //array [composite]
        
        return result.toString();
    }
    
    //Pre: addressConverter inicializada para direcciones de este SearchDescriptorNode.
    /**
     * Si el parmetro forExistence toma el valor false, entonces esto genera un cdigo
     * que comprueba los subelementos de los items de la lista "result" por los que NO
     * indexa el descriptor de ndices "id".
     * Si toma el valor true, lo que genera es un cdigo que comprueba dichos subelementos,
     * devolviendo true en cuanto encuentre uno que coincida. (para usar en el exists)
     */
    public String generateResultCleanupCode ( IndexDescriptor id , SearchDescriptorNode stn , boolean forExistence )
    {
    	
    	StringBuffer result = null; //devolveremos finalmente null si no hace falta limpieza.
    	
    	StringBuffer ifConditions = null; //condiciones de chequeo del if
    	
    	HashMap originalAddressConverter = new HashMap(); //convertidor de direcciones que nos viene dado
    	originalAddressConverter = (HashMap) addressConverter.clone();
    	
    	Iterator skelTreeAddresses = addressConverter.keySet().iterator();
    	List atomicIndices = id.getAtomicIndexes();
    	while ( skelTreeAddresses.hasNext() )
    	{
    		List address = (List) skelTreeAddresses.next();
    		
    		//no ground, composite o multimatcher: no hace falta chequeo.
    		//slo hace falta "cleanup" para los elementos simples ground no indexados.
    		Class nodeClass = stn.nodeAtAddress(address).getAssociatedClass();
    		if ( nodeClass == null || nodeClass.isAssignableFrom(SequenceElement.class) || nodeClass.isArray() )
    			continue;
    		
    		//buscamos si la direccin address aparece en el ndice id
    		boolean addressFound = false;
    		for ( int i = 0 ; i < atomicIndices.size() ; i++ )
    		{
    			AtomicIndex anAtomicIndex = (AtomicIndex) atomicIndices.get(i);
    			if ( anAtomicIndex.getTreeNodeAddress().equals(address) )
    			{
    				addressFound=true;
    				break;
    			}
    		}
    		
    		if ( !addressFound )
    		{
    			
    			if ( result == null ) //todava no tenemos nada en el cdigo de limpieza
    			{
    				result = new StringBuffer("");
    			    				
    				result.append("if ( result != null )");
    				result.append("{\n");
    				if ( !forExistence )
    					result.append("List cleanResult = new ArrayList();\n");
    				result.append("for ( int k = 0 ; k<result.size() ; k++ )\n");
    				result.append("{\n");
    				   				
    				//begin new cleanup check code
    				
    				String itemElementName = (String) originalAddressConverter.get ( address );
    				
    		    	StringBuffer decls = new StringBuffer("");
    		    	
    		    	//el doGenerateCode modifica el addressConverter.
    		    	StringBuffer code = new StringBuffer(doGenerateCode ( stn , new ArrayList() , "result.get(k)" , true , false , decls  ));
    		    	
    		    	String resultElementName = (String) addressConverter.get(address);
    		    	
    		    	result.append(decls);
    		    	result.append(code);
    				
    		    	
    				//red.
    		    	
    				result.append("if ( !compareItems(" + "(Object[])result.get(k)" + "," + "item" + "))\n");
    				result.append("skelmatch=false;\n");
    		    	 	
    		    	result.append("if ( skelmatch )\n");
    		    	result.append("{\n");
    		    	    		    	
    		    	ifConditions = new StringBuffer( "( " + itemElementName + ".equals( " + resultElementName + " ) )" );
    		    	
    				result.append("if ( %CONDITIONS% )\n");
    				if ( !forExistence )
    					result.append("cleanResult.add("+ "result.get(k)" + ");\n");
    				else
    					result.append("return true;\n");
    				
    				result.append("");
    				
    					    	
    		    	result.append("}\n");
    		    	result.append("skelmatch=true;\n");
    		    	

		
    				result.append("} //for\n");
    				if ( !forExistence )
    					result.append("result = cleanResult;\n");
    				result.append("} //if result not null\n"); 


    				
    			}
    			else
    			{
    				ifConditions.append ( " && ");
    				String itemElementName = (String) originalAddressConverter.get ( address );
    				String resultElementName = (String)addressConverter.get( address );
    				ifConditions.append( "( " + itemElementName + ".equals( " + resultElementName + " ) )" );
    			}
    			
    		}
    		
    		
    	} //while (addresses iterator loop)
    	
    	if ( result != null )
    		return (new String(result)).replaceAll("%CONDITIONS%",ifConditions.toString());
    	else return null;
    	
    }
  
    
}


