/*
 * Created at regulus on 06-nov-2004 18:44:36
 * as file StepHandlerCodeGenerator.java on package generator
 */
package generator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.List;

/**
 * @author carlos
 * Genera el cdigo para la clase StepHandler, que se hace cargo de la indexacin de pasos deductivos.
 *
 * Created at regulus, 06-nov-2004 18:44:36
 */
public class StepHandlerCodeGenerator extends ItemHandlerCodeGenerator
{
	
    protected String generateDeclarations ( IndexDescriptor[] indices )
    {
    	String s = super.generateDeclarations(indices);    	
        StringBuffer auxIndices = new StringBuffer();
        for ( int i = 0 ; i < indices.length ; i++ )
        {
        	auxIndices.append("private List " + indices[i].getName() + "_alt = new ArrayList();\n");
        }
        return s + auxIndices.toString();
    }
	
	public String generateCode ( SearchDescriptorNode[] skeletonStructures )
	{
		
		StringBuffer codeBuffer = new StringBuffer("");
		
		codeBuffer.append("package schema.step;\n\n");
        codeBuffer.append("import java.util.*;\n");
        codeBuffer.append("\n");
		codeBuffer.append("public class StepHandler\n");
		codeBuffer.append("{\n");
		
		codeBuffer.append("//Attributes:\n");
        
        //obtenemos los descriptores de los ndices que deberemos crear.
        IndexDescriptor[] indices = (IndexDescriptor[]) getSuitableIndices ( "stepIndex" , "stepList" , skeletonStructures ).toArray ( new IndexDescriptor[0]);
        
        codeBuffer.append(generateDeclarations(indices));
        codeBuffer.append("private static StepHandler instance;\n");
        codeBuffer.append("\n\n");
        
        codeBuffer.append("//Methods:\n");
        		
        codeBuffer.append("public StepHandler()\n");
        codeBuffer.append("{\n");
        codeBuffer.append(generateInitializations(indices));        
        codeBuffer.append("instance = this;\n");
        codeBuffer.append("}//[constructor]\n");
        
        codeBuffer.append("\n");
        
        codeBuffer.append("public static StepHandler instance()\n");
        codeBuffer.append("{\n");
        codeBuffer.append("return instance;\n");
        codeBuffer.append("}//[singleton access method]\n");
               
        codeBuffer.append("\n");
        
        codeBuffer.append("public List getAll()\n");
        codeBuffer.append("{\n");
        codeBuffer.append("return stepList;\n");
        codeBuffer.append("}//[getAll]\n");
		  
        codeBuffer.append( this.generateGetByItemMethod(indices,"item") );
        
        codeBuffer.append("\n");
        
        codeBuffer.append( this.generateGetSkeletonVariantsMethod() );
        
        codeBuffer.append("\n");
        
        codeBuffer.append( this.generateAddStepMethod(indices,"ds") );
        
        codeBuffer.append("\n");
        
        codeBuffer.append( this.generateAddStepSkeletonMethod(indices) );
        
        codeBuffer.append("\n\n} //[class]\n\n");
        
        return codeBuffer.toString();
	}
	
	
	
	private String generateGetByItemMethod ( IndexDescriptor[] indices , String objectName )
    {
        StringBuffer result = new StringBuffer("public List getByItem( Object[] " + objectName + ")\n");
        result.append("{\n");
        result.append("List result = null;\n");
        result.append("boolean skelmatch;\n");
        for ( int i = 0 ; i < indices.length ; i++ )
        {
            result.append(generateSinglePetitionCode(indices[i],objectName,true,false,false,true));
            result.append("if ( skelmatch ) return result;\n");
        }
        result.append("return null;\n");
        result.append("}//[getBySpecification]\n");
        return result.toString();
    }
	
	private String generateGetSkeletonVariantsMethod ()
	{
		StringBuffer result = new StringBuffer();
		try
		{
			//it's on a file.
			BufferedReader br = new BufferedReader ( new InputStreamReader ( new FileInputStream("common"+File.separator+"parts"+File.separator+"StepHandler.p1") ) );
			String linea;
			while ( (linea=br.readLine()) != null )
			{
				result.append(linea);
				result.append("\n");
			}
		}
		catch ( Exception ioe )
		{
			System.out.println("Problem when reading partial code file common"+File.separator+"parts"+File.separator+"StepHandler.p1:");
			ioe.printStackTrace();
		}
		return new String(result);
	}
	
	private String generateAddStepMethod ( IndexDescriptor[] indices , String objectName )
	{
		StringBuffer result = new StringBuffer("public void addStep ( DeductiveStep " + objectName + ")\n");
		result.append("{\n");
		result.append("List stepSkeletons = " + objectName + ".getAssociatedSpecifications();\n");
		result.append("for ( int i = 0 ; i < stepSkeletons.size() ; i++ )\n");
		result.append("{\n");
		result.append("Object[] aSkeleton = (Object[])stepSkeletons.get(i);\n");
		result.append("this.addStepSkeleton ( aSkeleton , " + objectName + " );\n");
		
		//BEGIN CODE ADDED 05.12.19
		
		//consider versions of the skeleton with subsequences removed
		result.append("List versions = getSkeletonVariants(aSkeleton);\n");
		result.append("for ( int j = 0 ; j < versions.size() ; j++ )\n");
		result.append("{;\n");
		result.append("Object[] skel = (Object[]) versions.get(j);\n");
		result.append("this.addStepSkeleton ( skel , " + objectName + " );\n");
		result.append("};\n");
		//this is a chapuz: the original is also added!
		
		//END CODE ADDED 05.12.19
		
		result.append("}//[foreach skeleton]\n");
		result.append("}//[addStep]\n");
		return result.toString();
	}
	
    /**
     * Genera el cdigo de aadir un objeto de nombre stepName al ndice dado, si procede.
     * (es decir, si el item de nombre objectName tiene un formato compatible con alguno 
     * de los skeleton trees asociados a ese ndice)
     * @param index ndice que estamos considerando.
     * @param objectName Nombre de la variable que contiene el item.
     * @param stepName Nombre de la variable que aadimos al ndice.
     * @return Cdigo que agrega la variable al ndice si forma parte del conjunto de items
     * que dicho ndice puede indexar.
     */
    protected String generateSingleAddCode ( IndexDescriptor index , String objectName , String stepName )
    {
        
        StringBuffer result = new StringBuffer("");
        
        List skeletonTrees = index.getAssociatedSearchDescriptors();
        
        
        for ( int i = 0 ; i < skeletonTrees.size() ; i++ )
        {
            SearchDescriptorNode currentSkeletonTree = (SearchDescriptorNode) skeletonTrees.get(i);
            
            SearchDescriptorMatchCodeGenerator smcg = new SearchDescriptorMatchCodeGenerator(objectName);
            
            result.append("skelmatch = true;\n");
            
            result.append( (smcg.generateCode(currentSkeletonTree,false,true)) );
            
            result.append("if ( skelmatch )\n");
            result.append("{\n");
            
            String indexUsageCode = index.getIndexUsageCode(smcg.getAddressConverter(),false);
            String initterCode = index.getIndexUsageCode(smcg.getAddressConverter(),true);
            
            List partialResults = index.getPartialIndexUsageCodeResults();
            
            result.append("try //indexing by null element?\n");
            result.append("{\n");
            
            for ( int j = 0 ; j < partialResults.size() ; j++ )
            {
            	IndexDescriptor.PartialAccessResult partialResult = (IndexDescriptor.PartialAccessResult) partialResults.get(j);
            	result.append ( partialResult.getDeclarationCode() + " partial" + j + ";\n");
            	result.append ( "if ( (partial" + j + "=" + partialResult.getAccessCode() + " ) == null )\n");
            	result.append ("{\n");
            	result.append ( partialResult.getInitializerCode() + ";\n");
            	result.append ( "partial"+j + " = " + partialResult.getAccessCode() + ";\n" );
            	result.append ("}\n");
            	//usar la variable parcial en el cdigo subsiguiente
            	indexUsageCode = indexUsageCode.replaceFirst("\\Q"+partialResult.getAccessCode()+"\\E","partial"+j);
            	for ( int k = j+1 ; k < partialResults.size() ; k++ )
            	{
            		((IndexDescriptor.PartialAccessResult)partialResults.get(i)).replaceAccessCode("\\Q"+partialResult.getAccessCode()+"\\E","partial"+j);
            	}
            }
            
            result.append("if ( " + indexUsageCode + " == null )\n");                          
            result.append(initterCode);
            
            result.append("if ( ("+ indexUsageCode + ").size() > 0 && (" + indexUsageCode + ").get(("+indexUsageCode+").size()-1) == " + stepName + ")");
            result.append("; //duplicate, do nothing!\n");
            result.append("else\n");
            
            result.append("(");
            result.append( indexUsageCode );
            result.append(").add("+stepName+");\n" );
            
            result.append("} //try\n");
            result.append("catch ( NullPointerException npe )\n");
            result.append("{\n");
            
            //[3 LOC added 2004-12-08]
            result.append("if ( ("+ index.getName() + "_alt" + ").size() > 0 && (" + index.getName() + "_alt" + ").get(("+index.getName() + "_alt"+").size()-1) == " + stepName + ")");
            result.append("; //duplicate, do nothing!\n");
            result.append("else\n");
            
            result.append( index.getName() + "_alt" + ".add(" + stepName + ");\n");
            
            result.append("}\n");
			
            
            result.append("}\n");
            
            if ( i < skeletonTrees.size() - 1 )
            {
                result.append("else\n");
                result.append("{\n");           
            }
        }
        //cerrar llaves abiertas, que son skeletonTrees.size() - 1 debido al ltimo if
        //del bucle anterior
        for ( int i = 0 ; i < skeletonTrees.size() - 1 ; i++ )
        {
            result.append("}\n");
        }
        
        if ( skeletonTrees.size() == 0 )
        {
            //caso especial sin ningn esqueleto asociado: es siempre el ndice vaco.
            //(lista de items)
            String indexUsageCode = index.getIndexUsageCode(new HashMap(),false);
            result.append("(");
            result.append( indexUsageCode );
            result.append(").add("+stepName+");\n");            
        }
        
        return result.toString();
        
    }
	
	private String generateAddStepSkeletonMethod ( IndexDescriptor[] indices )
	{
		StringBuffer result = new StringBuffer("");
		result.append("public void addStepSkeleton ( Object[] skeleton , DeductiveStep ds )\n");
		result.append("{\n");
		result.append("boolean skelmatch;\n");
		for ( int i = 0 ; i < indices.length ; i++ )
		{
			result.append(generateSingleAddCode(indices[i],"skeleton","ds"));
		}
		result.append("}//[addStepSkeleton]\n");
		return result.toString();
	}
	

	

}
