package grammar;

import element.*;

import java.util.*;
import java.io.*;

/**
 * Gramatica independiente del contexto. Es una lista de arrays de smbolos. Cada uno
 * de estos arrays de smbolos tiene como primer smbolo (ndice 0) el de la parte izquierda
 * de una produccin y como smbolos subsiguientes la parte derecha.
 *  
 * @author carlos
 *
 */
public class ContextFreeGrammar extends ArrayList implements Grammar
{
		
	/**
	 * Construye y devuelve la gramtica vaca.
	 *
	 */
	public ContextFreeGrammar ()
	{
		super();
		CFGUtils.registerGrammar(this);
	}
	
	public void initFromFile ( File f )
	{
		try
		{
			FileInputStream fin = new FileInputStream( f );
			BufferedReader br = new BufferedReader ( new InputStreamReader ( fin ));
			String linea;
			while ( (linea = br.readLine()) != null )
			{
				linea = linea.replace(':','%');
				StringTokenizer st = new StringTokenizer ( linea , ">" );
				/*PRECD: los smbolos > no pueden aparecer formando parte de
				 * smbolos de la gramtica.
				 */
				String s1 = st.nextToken();
				s1 = s1.substring(0,s1.length()-1);
				//lista donde almacenaremos temporalmente los smbolos de la regla
				ArrayList tempRule = new ArrayList();
				//smbolo de la parte izquierda de la regla
				Symbol leftHandSide = SymbolTable.instance().symbolFor(s1.trim());
				tempRule.add(leftHandSide);
				String s2 = null;
				try
				{
					s2 = st.nextToken();
				}
				catch ( Exception e ) 
				{ 
					//lado derecho vaco (regla epsilon)
					this.add(tempRule.toArray(new Symbol[0]));
					continue;	
				};
				StringTokenizer st2 = new StringTokenizer ( s2 , "()");
				String s3 = null;
				s3 = st2.nextToken();
				StringTokenizer st3;
				st3 = new StringTokenizer ( s3 , " " );
				//obtener o crear los smbolos de la parte derecha de la regla
				while ( st3.hasMoreTokens() )
				{
					tempRule.add ( SymbolTable.instance().symbolFor(st3.nextToken().trim()));					
				}
				//agregar esta regla a la gramtica, ya como array
				this.add(tempRule.toArray(new Symbol[0]));
			}
		}
		catch ( Throwable t )
		{
			System.err.println("Error reading grammar:");
			t.printStackTrace();
		}
	}
	
	/**
	 * Construye una gramtica a partir de un fichero en formato Susanne.
	 * @param f Fichero de donde se leer la gramtica (formato Susanne)
	 */
	public ContextFreeGrammar ( File f )
	{
		this();	
		initFromFile(f);
		CFGUtils.registerGrammar(this);
	}
	
	/**
	 * Devuelve una representacin humanamente legible de la gramtica.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer("");
		Iterator it = this.iterator();
		while ( it.hasNext() )
		{
			Symbol[] current = (Symbol[]) it.next();
			sb.append(SymbolTable.instance().getName(current[0]));
			sb.append(" -> ");
			for ( int i = 1 ; i < current.length ; i++ )
			{
				sb.append(SymbolTable.instance().getName(current[i]));
				sb.append(" ");
			}
			sb.append("\n");
		}
		return new String(sb);
	}
	
	//Pasa a forma normal de Chomsky. No es la implementacin ms ptima posible.
	public ContextFreeGrammar chomskyNormalForm()
	{
		Iterator git = this.iterator();
		ContextFreeGrammar result = new ContextFreeGrammar();
		
		int nameCounter = 0;
		Symbol[] rule = null;
		if ( git.hasNext() )
			rule = (Symbol[]) git.next();
		
		while ( rule != null )
		{
			if ( rule.length <= 3 ) //regla ya es CNF
			{
				result.add(rule);
				if ( git.hasNext() )
					rule = (Symbol[]) git.next();
				else
					rule = null;
				continue;
			}
			//regla no es cnf
			Symbol currentDummySymbol = SymbolTable.instance().symbolFor("CNF"+nameCounter);
			nameCounter++;
			Symbol[] binaryRule = new Symbol[3];
			binaryRule[0] = rule[0];
			binaryRule[1] = rule[1];
			binaryRule[2] = currentDummySymbol;
			result.add(binaryRule);
			//crear una siguiente regla "artificial" a procesar, metindola en rule
			Symbol[] newRule = new Symbol[rule.length-1];
			newRule[0] = currentDummySymbol;
			for ( int i = 2 ; i < rule.length ; i++ )
			{
				newRule[i-1]=rule[i];
			}
			rule = newRule;
		} //while rule not null
		
		return result;
		
	}
	
	public Element[] getEntity ( int i )
	{
		return (Element[])super.get(i);
	}
	
	
	
}