package grammar.tag.util;

import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import util.DOMNodeListWrapper;
import util.DOMNodeWrapper;

public class Binarizer extends AbstractGrammarTransformer
{
	
	public String toString()
	{
		return "Binarizing Grammar";
	}
	
	public Binarizer(String inFile, String outFile, AbstractGrammarTransformer next) 
	{
		super(inFile, outFile, next);
	}

	public Binarizer(String inFile, String outFile) 
	{
		super(inFile, outFile);
	}

	public Binarizer(AbstractGrammarTransformer next) 
	{
		super(next);
	}
	
	private int counter=0;
	public void preprocessTreeNode(DOMNodeWrapper treeNode)
	{
		counter = 0;
	}
	
	public void postprocessNodeNode(DOMNodeWrapper nodeNode)
	{
		if ( nodeNode.getDirectChildrenElementsByTagName("node").size() > 2 )
		{
			DOMNodeWrapper currentRoot = nodeNode;
			DOMNodeListWrapper pending = nodeNode.getDirectChildrenElementsByTagName("node");
			List pendingList = new LinkedList();
			for ( int i = 0 ; i < pending.size() ; i++ )
			{
				pendingList.add(pending.get(i));
			}
			
			//remove children
			for ( int i = 0 ; i < pending.size() ; i++ )
			{
				nodeNode.getNode().removeChild(pending.get(i).getNode());
			}
			
			//now re-add them in a convenient way
			while ( pendingList.size() > 2 )
			{
				//left link to first pending node
				currentRoot.getNode().appendChild(((DOMNodeWrapper)pendingList.get(0)).getNode());
				
				//create artificial node for right link
				Element e = nodeNode.getNode().getOwnerDocument().createElement("node");
				e.setAttribute("cat","bin");
				e.setAttribute("type","none");
				e.setAttribute("id","bin"+(counter++));
				
				//right link to artificial node
				currentRoot.getNode().appendChild(e);
				
				//re-set current root and pending list
				currentRoot = new DOMNodeWrapper(e);
				pendingList.remove(0);
			}
			
			//{assert: pendingList.size() == 2}
			currentRoot.getNode().appendChild(((DOMNodeWrapper)pendingList.get(0)).getNode());
			currentRoot.getNode().appendChild(((DOMNodeWrapper)pendingList.get(1)).getNode());
			
			
			
		}
			
	}

}
