/*
 * Created at regulus on 17-feb-2005 11:59:31
 * as file Main.java on package launcher
 */
package launcher;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.UnflaggedOption;
import com.martiansoftware.jsap.defaultsources.PropertyDefaultSource;
import com.martiansoftware.jsap.stringparsers.StringStringParser;

import parser.sparser.SchemaParser;

/**
 * @author carlos
 *
 * Clase cuya nica funcin es lanzar el "schema parser", es decir, proporcionar el
 * mtodo main, leyendo los argumentos de lnea de comandos.
 */
public class Main 
{

	public static void main ( String[] args )
	{
		
		JSAP argumentParser = new JSAP();
		
		UnflaggedOption schemaFileOption = new UnflaggedOption ( "schemaFile" , new StringStringParser() , null , true , false );
		schemaFileOption.setHelp("Fichero que contiene la especificacion de un esquema de analisis sintactico.");
		
		FlaggedOption outputDirectoryOption = new FlaggedOption ( "outputDir" , new StringStringParser() , "generated" + File.separator , false , 'o' , JSAP.NO_LONGFLAG );
		outputDirectoryOption.setHelp("Ruta absoluta o relativa al directorio en que se escribira el codigo generado. Por defecto se usara generated/");
		
		FlaggedOption grammarClassOption = new FlaggedOption ( "gClass" , new StringStringParser() , "grammar.ContextFreeGrammar" , false , 'g' , "def-gram-class" );
		grammarClassOption.setHelp("Nombre completo de la clase de gramaticas que se utilizara en los esquemas que no especifiquen una. Por defecto se usara grammar.ContextFreeGrammar");
		
		try
		{
			argumentParser.registerParameter(schemaFileOption);
			argumentParser.registerParameter(outputDirectoryOption);
			argumentParser.registerParameter(grammarClassOption);
			argumentParser.registerDefaultSource(new PropertyDefaultSource("config.properties",false));
		}
		catch ( JSAPException exc ) //esto no debera suceder.
		{
			System.out.println("Error interno:");
			exc.printStackTrace();
		}
		
		JSAPResult config = argumentParser.parse(args);
		
		if ( !config.success() )
		{
			System.err.println();
			for (java.util.Iterator errs = config.getErrorMessageIterator(); errs.hasNext();) 
				System.err.println("Error: " + errs.next());
			System.err.println();
            System.err.print("Uso: java "
                                + Main.class.getName());
            System.err.println(" "
                                + argumentParser.getUsage());
            System.err.println();
            System.err.println(argumentParser.getHelp());
            System.err.println("Notese que los parametros opcionales que no se especifiquen tomaran el valor que aparezca en el fichero config.properties, y solo si no existe este fichero o no contiene la opcion se utilizara el valor por defecto especificado.");
            System.exit(1);
		}
		
		String schemaFile = config.getString("schemaFile");
		System.out.println("Schema File: "+schemaFile);
		
		try
		{
			System.setIn(new FileInputStream(schemaFile));
		}
		catch ( FileNotFoundException fnfe )
		{
			System.out.println("El fichero de definicin de esquema no se ha encontrado:");
			fnfe.printStackTrace();
			System.exit(-1);
		}
		
		try
		{
			SchemaParser.setOutputDirectory(config.getString("outputDir"));
			SchemaParser.setDefaultGrammarClassName(config.getString("gClass"));
			//SchemaParser.setVerbose()
			SchemaParser.main(new String[0]);
		}
		catch ( Exception e )
		{
			e.printStackTrace();
			System.exit(-1);
		}
		
	}
	

}
